#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, 2019 Kevin Breit (@kbreit) <kevin.breit@kevinbreit.net>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["deprecated"],
    "supported_by": "community",
}

DOCUMENTATION = r"""
author:
  - Kevin Breit (@kbreit)
deprecated:
  alternative: cisco.meraki.networks_appliance_static_routes
  removed_in: 3.0.0
  why: Updated modules released with increased functionality
description:
  - Allows for creation, management, and visibility into static routes within Meraki.
extends_documentation_fragment: cisco.meraki.meraki
module: meraki_mx_static_route
options:
  enabled:
    description:
      - Indicates whether static route is enabled within a network.
    type: bool
  fixed_ip_assignments:
    description:
      - List of fixed MAC to IP bindings for DHCP.
    elements: dict
    suboptions:
      ip:
        description:
          - IP address of endpoint.
        type: str
      mac:
        description:
          - MAC address of endpoint.
        type: str
      name:
        description:
          - Hostname of endpoint.
        type: str
    type: list
  gateway_ip:
    description:
      - IP address of the gateway for the subnet.
    type: str
  gateway_vlan_id:
    description:
      - The gateway IP (next hop) VLAN ID of the static route.
    type: int
  name:
    description:
      - Descriptive name of the static route.
    type: str
  net_id:
    description:
      - ID number of a network.
    type: str
  net_name:
    description:
      - Name of a network.
    type: str
  reserved_ip_ranges:
    description:
      - List of IP ranges reserved for static IP assignments.
    elements: dict
    suboptions:
      comment:
        description:
          - Human readable description of reservation range.
        type: str
      end:
        description:
          - Last IP address of reserved range.
        type: str
      start:
        description:
          - First IP address of reserved range.
        type: str
    type: list
  route_id:
    description:
      - Unique ID of static route.
    type: str
  state:
    choices:
      - absent
      - query
      - present
    default: present
    description:
      - Create or modify an organization.
    type: str
  subnet:
    description:
      - CIDR notation based subnet for static route.
    type: str
short_description: Manage static routes in the Meraki cloud
"""

EXAMPLES = r"""
- name: Create static_route
  meraki_static_route:
    auth_key: abc123
    state: present
    org_name: YourOrg
    net_name: YourNet
    name: Test Route
    subnet: 192.0.1.0/24
    gateway_ip: 192.168.128.1
  delegate_to: localhost
- name: Update static route with fixed IP assignment
  meraki_static_route:
    auth_key: abc123
    state: present
    org_name: YourOrg
    net_name: YourNet
    route_id: d6fa4821-1234-4dfa-af6b-ae8b16c20c39
    fixed_ip_assignments:
      - mac: aa:bb:cc:dd:ee:ff
        ip: 192.0.1.11
        comment: Server
  delegate_to: localhost
- name: Query static routes
  meraki_static_route:
    auth_key: abc123
    state: query
    org_name: YourOrg
    net_name: YourNet
  delegate_to: localhost
- name: Delete static routes
  meraki_static_route:
    auth_key: abc123
    state: absent
    org_name: YourOrg
    net_name: YourNet
    route_id: '{{ item }}'
  delegate_to: localhost
"""

RETURN = r"""
data:
  description: Information about the created or manipulated object.
  returned: info
  type: complex
  contains:
    id:
      description: Unique identification string assigned to each static route.
      returned: success
      type: str
      sample: d6fa4821-1234-4dfa-af6b-ae8b16c20c39
    net_id:
      description: Identification string of network.
      returned: query or update
      type: str
      sample: N_12345
    name:
      description: Name of static route.
      returned: success
      type: str
      sample: Data Center static route
    subnet:
      description: CIDR notation subnet for static route.
      returned: success
      type: str
      sample: 192.0.1.0/24
    gatewayIp:
      description: Next hop IP address.
      returned: success
      type: str
      sample: 192.1.1.1
    enabled:
      description: Enabled state of static route.
      returned: query or update
      type: bool
      sample: true
    reservedIpRanges:
      description: List of IP address ranges which are reserved for static assignment.
      returned: query or update
      type: complex
      contains:
        start:
          description: First address in reservation range, inclusive.
          returned: query or update
          type: str
          sample: 192.0.1.2
        end:
          description: Last address in reservation range, inclusive.
          returned: query or update
          type: str
          sample: 192.0.1.10
        comment:
          description: Human readable description of range.
          returned: query or update
          type: str
          sample: Server range
    fixedIpAssignments:
      description: List of static MAC to IP address bindings.
      returned: query or update
      type: complex
      contains:
        mac:
          description: Key is MAC address of endpoint.
          returned: query or update
          type: complex
          contains:
            ip:
              description: IP address to be bound to the endpoint.
              returned: query or update
              type: str
              sample: 192.0.1.11
            name:
              description: Hostname given to the endpoint.
              returned: query or update
              type: str
              sample: JimLaptop
"""

from ansible.module_utils.basic import AnsibleModule, json
from ansible_collections.cisco.meraki.plugins.module_utils.network.meraki.meraki import (
    MerakiModule,
    meraki_argument_spec,
)


def fixed_ip_factory(meraki, data):
    fixed_ips = dict()
    for item in data:
        fixed_ips[item["mac"]] = {"ip": item["ip"], "name": item["name"]}
    return fixed_ips


def get_static_routes(meraki, net_id):
    path = meraki.construct_path("get_all", net_id=net_id)
    r = meraki.request(path, method="GET")
    return r


def get_static_route(meraki, net_id, route_id):
    path = meraki.construct_path(
        "get_one", net_id=net_id, custom={"route_id": route_id}
    )
    r = meraki.request(path, method="GET")
    return r


def does_route_exist(name, routes):
    for route in routes:
        if name == route["name"]:
            return route
    return None


def update_dict(original, proposed):
    for k, v in proposed.items():
        if v is not None:
            original[k] = v
    return original


def main():

    # define the available arguments/parameters that a user can pass to
    # the module

    fixed_ip_arg_spec = dict(
        mac=dict(type="str"),
        ip=dict(type="str"),
        name=dict(type="str"),
    )

    reserved_ip_arg_spec = dict(
        start=dict(type="str"),
        end=dict(type="str"),
        comment=dict(type="str"),
    )

    argument_spec = meraki_argument_spec()
    argument_spec.update(
        net_id=dict(type="str"),
        net_name=dict(type="str"),
        name=dict(type="str"),
        subnet=dict(type="str"),
        gateway_ip=dict(type="str"),
        state=dict(
            type="str", default="present", choices=["absent", "present", "query"]
        ),
        fixed_ip_assignments=dict(
            type="list", elements="dict", options=fixed_ip_arg_spec
        ),
        reserved_ip_ranges=dict(
            type="list", elements="dict", options=reserved_ip_arg_spec
        ),
        route_id=dict(type="str"),
        enabled=dict(type="bool"),
        gateway_vlan_id=dict(type="int"),
    )

    # the AnsibleModule object will be our abstraction working with Ansible
    # this includes instantiation, a couple of common attr would be the
    # args/params passed to the execution, as well as if the module
    # supports check mode
    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    meraki = MerakiModule(module, function="static_route")
    module.params["follow_redirects"] = "all"
    payload = None

    query_urls = {"static_route": "/networks/{net_id}/appliance/staticRoutes"}
    query_one_urls = {
        "static_route": "/networks/{net_id}/appliance/staticRoutes/{route_id}"
    }
    create_urls = {"static_route": "/networks/{net_id}/appliance/staticRoutes/"}
    update_urls = {
        "static_route": "/networks/{net_id}/appliance/staticRoutes/{route_id}"
    }
    delete_urls = {
        "static_route": "/networks/{net_id}/appliance/staticRoutes/{route_id}"
    }
    meraki.url_catalog["get_all"].update(query_urls)
    meraki.url_catalog["get_one"].update(query_one_urls)
    meraki.url_catalog["create"] = create_urls
    meraki.url_catalog["update"] = update_urls
    meraki.url_catalog["delete"] = delete_urls

    if not meraki.params["org_name"] and not meraki.params["org_id"]:
        meraki.fail_json(
            msg="Parameters 'org_name' or 'org_id' parameters are required"
        )
    if not meraki.params["net_name"] and not meraki.params["net_id"]:
        meraki.fail_json(
            msg="Parameters 'net_name' or 'net_id' parameters are required"
        )
    if meraki.params["net_name"] and meraki.params["net_id"]:
        meraki.fail_json(msg="'net_name' and 'net_id' are mutually exclusive")

    # Construct payload
    if meraki.params["state"] == "present":
        payload = dict()
        if meraki.params["net_name"]:
            payload["name"] = meraki.params["net_name"]

    # manipulate or modify the state as needed (this is going to be the
    # part where your module will do what it needs to do)

    org_id = meraki.params["org_id"]
    if not org_id:
        org_id = meraki.get_org_id(meraki.params["org_name"])
    net_id = meraki.params["net_id"]
    if net_id is None:
        nets = meraki.get_nets(org_id=org_id)
        net_id = meraki.get_net_id(net_name=meraki.params["net_name"], data=nets)

    if meraki.params["state"] == "query":
        if meraki.params["route_id"] is not None:
            meraki.result["data"] = get_static_route(
                meraki, net_id, meraki.params["route_id"]
            )
        else:
            meraki.result["data"] = get_static_routes(meraki, net_id)
    elif meraki.params["state"] == "present":
        payload = {
            "name": meraki.params["name"],
            "subnet": meraki.params["subnet"],
            "gatewayIp": meraki.params["gateway_ip"],
        }
        if meraki.params["fixed_ip_assignments"] is not None:
            payload["fixedIpAssignments"] = fixed_ip_factory(
                meraki, meraki.params["fixed_ip_assignments"]
            )
        if meraki.params["reserved_ip_ranges"] is not None:
            payload["reservedIpRanges"] = meraki.params["reserved_ip_ranges"]
        if meraki.params["enabled"] is not None:
            payload["enabled"] = meraki.params["enabled"]
        if meraki.params["gateway_vlan_id"] is not None:
            payload["gatewayVlanId"] = meraki.params["gateway_vlan_id"]

        route_id = meraki.params["route_id"]
        if meraki.params["name"] is not None and route_id is None:
            route_status = does_route_exist(
                meraki.params["name"], get_static_routes(meraki, net_id)
            )
            if route_status is not None:  # Route exists, assign route_id
                route_id = route_status["id"]

        if route_id is not None:
            existing_route = get_static_route(meraki, net_id, route_id)
            original = existing_route.copy()
            payload = update_dict(existing_route, payload)
            if module.check_mode:
                meraki.result["data"] = payload
                meraki.exit_json(**meraki.result)
            if meraki.is_update_required(original, payload, optional_ignore=["id"]):
                path = meraki.construct_path(
                    "update", net_id=net_id, custom={"route_id": route_id}
                )
                meraki.result["data"] = meraki.request(
                    path, method="PUT", payload=json.dumps(payload)
                )
                meraki.result["changed"] = True
            else:
                meraki.result["data"] = original
        else:
            if module.check_mode:
                meraki.result["data"] = payload
                meraki.exit_json(**meraki.result)
            path = meraki.construct_path("create", net_id=net_id)
            meraki.result["data"] = meraki.request(
                path, method="POST", payload=json.dumps(payload)
            )
            meraki.result["changed"] = True
    elif meraki.params["state"] == "absent":
        if module.check_mode:
            meraki.exit_json(**meraki.result)
        path = meraki.construct_path(
            "delete", net_id=net_id, custom={"route_id": meraki.params["route_id"]}
        )
        meraki.result["data"] = meraki.request(path, method="DELETE")
        meraki.result["changed"] = True

    # in the event of a successful module execution, you will want to
    # simple AnsibleModule.exit_json(), passing the key/value results
    meraki.exit_json(**meraki.result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: ec2_transit_gateway_vpc_attachment_info
short_description: describes AWS Transit Gateway VPC attachments
version_added: 4.0.0
version_added_collection: community.aws
description:
  - Describes AWS Transit Gateway VPC Attachments.
options:
  id:
    description:
      - The ID of the Transit Gateway Attachment.
      - Mutually exclusive with O(name) and O(filters).
    type: str
    required: false
    aliases: ["attachment_id"]
  name:
    description:
      - The V(Name) tag of the Transit Gateway attachment.
    type: str
    required: false
  filters:
    description:
      - A dictionary of filters to apply. Each dict item consists of a filter key and a filter value.
      - Setting a V(tag:Name) filter will override the O(name) parameter.
    type: dict
    required: false
  include_deleted:
    description:
      - If O(include_deleted=True), then attachments in a deleted state will
        also be returned.
      - Setting a V(state) filter will override the O(include_deleted) parameter.
    type: bool
    required: false
    default: false
author:
  - Mark Chappell (@tremble)
  - Alina Buzachis (@alinabuzachis)
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: Describe a specific Transit Gateway attachment
  amazon.aws.ec2_transit_gateway_vpc_attachment_info:
    id: "tgw-attach-0123456789abcdef0"

- name: Describe all attachments attached to a transit gateway
  amazon.aws.ec2_transit_gateway_vpc_attachment_info:
    filters:
      transit-gateway-id: "tgw-0fedcba9876543210"

- name: Describe all attachments in an account
  amazon.aws.ec2_transit_gateway_vpc_attachment_info:
"""

RETURN = r"""
attachments:
  description: The attributes of the Transit Gateway attachments.
  type: list
  elements: dict
  returned: success
  contains:
    creation_time:
      description:
        - An ISO 8601 date time stamp of when the attachment was created.
      type: str
      returned: success
      sample: "2022-03-10T16:40:26+00:00"
    options:
      description:
        - Additional VPC attachment options.
      type: dict
      returned: success
      contains:
        appliance_mode_support:
          description:
            - Indicates whether appliance mode support is enabled.
          type: str
          returned: success
          sample: "enable"
        dns_support:
          description:
            - Indicates whether DNS support is enabled.
          type: str
          returned: success
          sample: "disable"
        ipv6_support:
          description:
            - Indicates whether IPv6 support is disabled.
          type: str
          returned: success
          sample: "disable"
        security_group_referencing_support:
          description:
            - Indicated weather security group referencing support is disabled.
          type: str
          returned: success
          sample: "enable"
    state:
      description:
        - The state of the attachment.
      type: str
      returned: success
      sample: "deleting"
    subnet_ids:
      description:
        - The IDs of the subnets in use by the attachment.
      type: list
      elements: str
      returned: success
      sample: ["subnet-0123456789abcdef0", "subnet-11111111111111111"]
    tags:
      description:
        - A dictionary representing the resource tags.
      type: dict
      returned: success
    transit_gateway_attachment_id:
      description:
        - The ID of the attachment.
      type: str
      returned: success
      sample: "tgw-attach-0c0c5fd0b0f01d1c9"
    transit_gateway_id:
      description:
        - The ID of the transit gateway that the attachment is connected to.
      type: str
      returned: success
      sample: "tgw-0123456789abcdef0"
    vpc_id:
      description:
        - The ID of the VPC that the attachment is connected to.
      type: str
      returned: success
      sample: "vpc-0123456789abcdef0"
    vpc_owner_id:
      description:
        - The ID of the account that the VPC belongs to.
      type: str
      returned: success
      sample: "123456789012"
"""

from typing import Any
from typing import Dict
from typing import List

from ansible_collections.amazon.aws.plugins.module_utils.ec2 import AnsibleEC2Error
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_transit_gateway_vpc_attachments
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.transformation import ansible_dict_to_boto3_filter_list
from ansible_collections.amazon.aws.plugins.module_utils.transformation import boto3_resource_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.transitgateway import get_states


def main():
    argument_spec = dict(
        id=dict(type="str", required=False, aliases=["attachment_id"]),
        name=dict(type="str", required=False),
        filters=dict(type="dict", required=False),
        include_deleted=dict(type="bool", required=False, default=False),
    )

    mutually_exclusive = [
        ["id", "name"],
        ["id", "filters"],
    ]

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        mutually_exclusive=mutually_exclusive,
    )

    name = module.params.get("name")
    attachment_id = module.params.get("id")
    opt_filters = module.params.get("filters")
    include_deleted = module.params.get("include_deleted")

    client = module.client("ec2")

    params: Dict[str, Any] = {}
    filters: Dict[str, Any] = {}
    attachments: List[Dict[str, Any]] = []

    if attachment_id:
        params["TransitGatewayAttachmentIds"] = [attachment_id]

    # Add filter by name if provided
    if name:
        filters["tag:Name"] = name

    # Include only active states if "include_deleted" is False
    if not include_deleted:
        filters["state"] = get_states()

    # Include any additional filters provided by the user
    if opt_filters:
        filters.update(opt_filters)

    if filters:
        params["Filters"] = ansible_dict_to_boto3_filter_list(filters)

    try:
        result = describe_transit_gateway_vpc_attachments(client, **params)
    except AnsibleEC2Error as e:
        module.fail_json_aws_error(e)

    if result:
        attachments = [boto3_resource_to_ansible_dict(attachment) for attachment in result]

    module.exit_json(changed=False, attachments=attachments, filters=filters)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

# (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
name: aws_account_attribute
author:
  - Sloane Hertel (@s-hertel) <shertel@redhat.com>
short_description: Look up AWS account attributes
description:
  - Describes attributes of your AWS account. You can specify one of the listed
    attribute choices or omit it to see all attributes.
options:
  attribute:
    description: The attribute for which to get the value(s).
    choices:
      - supported-platforms
      - default-vpc
      - max-instances
      - vpc-max-security-groups-per-interface
      - max-elastic-ips
      - vpc-max-elastic-ips
      - has-ec2-classic
extends_documentation_fragment:
  - amazon.aws.boto3
  - amazon.aws.common.plugins
  - amazon.aws.region.plugins
"""

EXAMPLES = r"""
vars:
  has_ec2_classic: "{{ lookup('aws_account_attribute', attribute='has-ec2-classic') }}"
  # true | false

  default_vpc_id: "{{ lookup('aws_account_attribute', attribute='default-vpc') }}"
  # vpc-xxxxxxxx | none

  account_details: "{{ lookup('aws_account_attribute', wantlist='true') }}"
  # {'default-vpc': ['vpc-xxxxxxxx'], 'max-elastic-ips': ['5'], 'max-instances': ['20'],
  #  'supported-platforms': ['VPC', 'EC2'], 'vpc-max-elastic-ips': ['5'], 'vpc-max-security-groups-per-interface': ['5']}
"""

RETURN = r"""
_raw:
  description:
    - Returns a boolean when O(attribute=check_ec2_classic). Otherwise returns the value(s) of the attribute
      (or all attributes if one is not specified).
"""

try:
    import botocore
except ImportError:
    pass  # Handled by AWSLookupBase

from ansible.errors import AnsibleLookupError
from ansible.module_utils.common.text.converters import to_native

from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.plugin_utils.lookup import AWSLookupBase


def _describe_account_attributes(client, **params):
    return client.describe_account_attributes(aws_retry=True, **params)


class LookupModule(AWSLookupBase):
    def run(self, terms, variables, **kwargs):
        super().run(terms, variables, **kwargs)

        client = self.client("ec2", AWSRetry.jittered_backoff())

        attribute = kwargs.get("attribute")
        params = {"AttributeNames": []}
        check_ec2_classic = False
        if "has-ec2-classic" == attribute:
            check_ec2_classic = True
            params["AttributeNames"] = ["supported-platforms"]
        elif attribute:
            params["AttributeNames"] = [attribute]

        try:
            response = _describe_account_attributes(client, **params)["AccountAttributes"]
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            raise AnsibleLookupError(f"Failed to describe account attributes: {to_native(e)}")

        if check_ec2_classic:
            attr = response[0]
            return [any(value["AttributeValue"] == "EC2" for value in attr["AttributeValues"])]

        if attribute:
            attr = response[0]
            return [value["AttributeValue"] for value in attr["AttributeValues"]]

        flattened = {}
        for k_v_dict in response:
            flattened[k_v_dict["AttributeName"]] = [value["AttributeValue"] for value in k_v_dict["AttributeValues"]]
        return [flattened]



# -*- coding: utf-8 -*-

# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
name: aws_ec2
short_description: EC2 inventory source
extends_documentation_fragment:
  - inventory_cache
  - constructed
  - amazon.aws.boto3
  - amazon.aws.common.plugins
  - amazon.aws.region.plugins
  - amazon.aws.assume_role.plugins
description:
  - Get inventory hosts from Amazon Web Services EC2.
  - The inventory file is a YAML configuration file and must end with C(aws_ec2.{yml|yaml}). For example - C(my_inventory.aws_ec2.yml).
notes:
  - If no credentials are provided and the control node has an associated IAM instance profile then the
    role will be used for authentication.
  - The C(tags) host variable is deprecated and will be removed in a release after 2026-12-01.
    Use C(ec2_tags) instead to avoid conflicts with Ansible reserved variable names.
  - The C(ec2_tags) host variable was added in version 11.2.0.
  - The C(use_contrib_script_compatible_ec2_tag_keys) option is deprecated and will be removed in a release after 2026-12-01.
    Use the C(ec2_tags) structure instead (e.g. use C(ec2_tags.TAGNAME) rather than C(ec2_tag_TAGNAME)).
  - The C(use_contrib_script_compatible_sanitization) option is deprecated and will be removed in a release after 2026-12-01.
    Use Ansible's default group name sanitization instead.
author:
  - Sloane Hertel (@s-hertel)
options:
  regions:
    description:
      - A list of regions in which to describe EC2 instances.
      - If empty (the default) default this will include all regions, except possibly restricted ones like V(us-gov-west-1) and V(cn-north-1).
    type: list
    elements: str
    default: []
  hostnames:
    description:
      - A list in order of precedence for hostname variables.
      - The elements of the list can be a dict with the keys mentioned below or a string.
      - Can be one of the options specified in U(http://docs.aws.amazon.com/cli/latest/reference/ec2/describe-instances.html#options).
      - If value provided does not exist in the above options, it will be used as a literal string.
      - To use tags as hostnames use the syntax tag:Name=Value to use the hostname Name_Value, or tag:Name to use the value of the Name tag.
      - Jinja2 filters can be added to the hostnames string. Added in version 9.2.0.
    type: list
    elements: raw
    default: []
    suboptions:
      name:
        description:
          - Name of the host.
        type: str
        required: true
      prefix:
        description:
          - Prefix to prepend to O(hostnames.name). Same options as O(hostnames.name).
          - If O(hostnames.prefix) is specified, final hostname will be O(hostnames.prefix) + O(hostnames.separator) + O(hostnames.name).
        type: str
        default: ''
        required: false
      separator:
        description:
          - Value to separate O(hostnames.prefix) and O(hostnames.name) when O(hostnames.prefix) is specified.
        type: str
        default: '_'
        required: false
  allow_duplicated_hosts:
    description:
      - By default, the first name that matches an entry of the O(hostnames) list is returned.
      - Turn this flag on if you don't mind having duplicated entries in the inventory
        and you want to get all the hostnames that match.
    type: bool
    default: false
    version_added: 5.0.0
  filters:
    description:
      - A dictionary of filter value pairs.
      - Available filters are listed here U(http://docs.aws.amazon.com/cli/latest/reference/ec2/describe-instances.html#options).
    type: dict
    default: {}
  include_filters:
    description:
      - A list of filters. Any instances matching at least one of the filters are included in the result.
      - Available filters are listed here U(http://docs.aws.amazon.com/cli/latest/reference/ec2/describe-instances.html#options).
      - Every entry in this list triggers a search query. As such, from a performance point of view, it's better to
        keep the list as short as possible.
    type: list
    elements: dict
    default: []
    version_added: 1.5.0
  exclude_filters:
    description:
      - A list of filters. Any instances matching one of the filters are excluded from the result.
      - The filters from O(exclude_filters) take priority over the O(include_filters) and O(filters) keys.
      - Available filters are listed here U(http://docs.aws.amazon.com/cli/latest/reference/ec2/describe-instances.html#options).
      - Every entry in this list triggers a search query. As such, from a performance point of view, it's better to
        keep the list as short as possible.
    type: list
    elements: dict
    default: []
    version_added: 1.5.0
  strict_permissions:
    description:
      - By default if a 403 (Forbidden) error code is encountered this plugin will fail.
      - You can set this option to False in the inventory config file which will allow 403 errors to be gracefully skipped.
    type: bool
    default: true
  use_contrib_script_compatible_sanitization:
    description:
      - By default this plugin is using a general group name sanitization to create safe and usable group names for use in Ansible.
        This option allows you to override that, in efforts to allow migration from the old C(ec2.py) inventory script and
        matches the sanitization of groups when the script's ``replace_dash_in_groups`` option is set to ``False``.
        To replicate behavior of ``replace_dash_in_groups = True`` with constructed groups,
        you will need to replace hyphens with underscores via the regex_replace filter for those entries.
      - For this to work you should also turn off the TRANSFORM_INVALID_GROUP_CHARS setting,
        otherwise the core engine will just use the standard sanitization on top.
      - This is not the default as such names break certain functionality as not all characters are valid Python identifiers
        which group names end up being used as.
      - The use of this feature is deprecated and will be removed in a release after 2026-12-01.
        Use Ansible's default group name sanitization instead.
    type: bool
    default: false
  use_contrib_script_compatible_ec2_tag_keys:
    description:
      - Expose the host tags with C(ec2_tag_TAGNAME) keys like the old C(ec2.py) inventory script.
      - The use of this feature is deprecated and will be removed in a release after 2026-12-01.
        Use the C(ec2_tags) structure instead (e.g. use C(ec2_tags.TAGNAME) rather than C(ec2_tag_TAGNAME)).
    type: bool
    default: false
    version_added: 1.5.0
  hostvars_prefix:
    description:
      - The prefix for host variables names coming from AWS.
    type: str
    version_added: 3.1.0
  hostvars_suffix:
    description:
      - The suffix for host variables names coming from AWS.
    type: str
    version_added: 3.1.0
  use_ssm_inventory:
    description:
      - Enables fetching additional EC2 instance information from the AWS Systems Manager (SSM) inventory service into hostvars.
      - By leveraging the SSM inventory data, the O(use_ssm_inventory) option provides additional details and attributes
        about the EC2 instances in your inventory. These details can include operating system information, installed software,
        network configurations, and custom inventory attributes defined in SSM.
    type: bool
    default: false
    version_added: 6.0.0
  route53_enabled:
    description:
      - Whether or not to use the Route53 DNS as host name.
      - When set to V(true), Route53 hostnames will be included.
    type: bool
    default: false
    version_added: 10.1.0
  route53_hostnames:
    description:
      - Specifies the Route53 DNS name suffix to consider.
      - Consider only Route53 zone with name ending with the value specified here.
    type: list
    elements: str
    default: []
    version_added: 10.1.0
  route53_excluded_zones:
    description:
      - A list of Route53 zones to exclude.
    type: list
    elements: str
    default: []
    version_added: 10.1.0
"""

EXAMPLES = r"""
# Minimal example using environment vars or instance role credentials
# Fetch all hosts in us-east-1, the hostname is the public DNS if it exists, otherwise the private IP address
plugin: amazon.aws.aws_ec2
regions:
  - us-east-1

---

# Example using filters, ignoring permission errors, and specifying the hostname precedence
plugin: amazon.aws.aws_ec2
# The values for profile, access key, secret key and token can be hardcoded like:
profile: aws_profile
# or you could use Jinja as:
# profile: "{{ lookup('env', 'AWS_PROFILE') | default('aws_profile', true) }}"
# Populate inventory with instances in these regions
regions:
  - us-east-1
  - us-east-2
filters:
  ## All instances with their `Environment` tag set to `dev`
  # tag:Environment: dev

  # All dev and QA hosts
  tag:Environment:
    - dev
    - qa
  instance.group-id: sg-xxxxxxxx
# Ignores 403 errors rather than failing
strict_permissions: false
# Note: I(hostnames) sets the inventory_hostname. To modify ansible_host without modifying
# inventory_hostname use compose (see example below).
hostnames:
  - tag:Name=Tag1,Name=Tag2  # Return specific hosts only
  - tag:CustomDNSName
  - dns-name
  - name: 'tag:Name=Tag1,Name=Tag2'
  - name: 'private-ip-address'
    separator: '_'
    prefix: 'tag:Name'
  - name: 'test_literal' # Using literal values for hostname
    separator: '-'       # Hostname will be aws-test_literal
    prefix: 'aws'

# Returns all the hostnames for a given instance
allow_duplicated_hosts: false

---

# Example using constructed features to create groups and set ansible_host
plugin: amazon.aws.aws_ec2
regions:
  - us-east-1
  - us-west-1
# keyed_groups may be used to create custom groups
strict: false
keyed_groups:
  # Add e.g. x86_64 hosts to an arch_x86_64 group
  - prefix: arch
    key: 'architecture'
  # Add hosts to tag_Name_Value groups for each Name/Value tag pair
  - prefix: tag
    key: ec2_tags
  # Add hosts to e.g. instance_type_z3_tiny
  - prefix: instance_type
    key: instance_type
  # Create security_groups_sg_abcd1234 group for each SG
  - key: 'security_groups|json_query("[].group_id")'
    prefix: 'security_groups'
  # Create a group for each value of the Application tag
  - key: ec2_tags.Application
    separator: ''
  # Create a group per region e.g. aws_region_us_east_2
  - key: placement.region
    prefix: aws_region
  # Create a group (or groups) based on the value of a custom tag "Role" and add them to a metagroup called "project"
  - key: ec2_tags['Role']
    prefix: foo
    parent_group: "project"
# Set individual variables with compose
compose:
  # Use the private IP address to connect to the host
  # (note: this does not modify inventory_hostname, which is set via I(hostnames))
  ansible_host: private_ip_address

---

# Example using include_filters and exclude_filters to compose the inventory.
plugin: amazon.aws.aws_ec2
regions:
  - us-east-1
  - us-west-1
include_filters:
  - tag:Name:
      - 'my_second_tag'
  - tag:Name:
      - 'my_third_tag'
exclude_filters:
  - tag:Name:
      - 'my_first_tag'

---

# Example using groups to assign the running hosts to a group based on vpc_id
plugin: amazon.aws.aws_ec2
profile: aws_profile
# Populate inventory with instances in these regions
regions:
  - us-east-2
filters:
  # All instances with their state as `running`
  instance-state-name: running
keyed_groups:
  - prefix: tag
    key: ec2_tags
compose:
  ansible_host: public_dns_name
groups:
  libvpc: vpc_id == 'vpc-####'

---

# Define prefix and suffix for host variables coming from AWS.
plugin: amazon.aws.aws_ec2
regions:
  - us-east-1
hostvars_prefix: 'aws_'
hostvars_suffix: '_ec2'

---

# Define hostnames variables with jinja2 filters.
plugin: amazon.aws.aws_ec2
regions:
  - us-east-1
hostnames:
  - "tag:Name | replace('test', 'prod')"

---

# Define inventory with Route53 enabled and filtering specific route53 host names suffix.
plugin: amazon.aws.aws_ec2
regions:
  - us-east-1
route53_enabled: true
route53_hostnames:
  - .example.com
  - .another_example.org

---

# Define inventory with Route53 enabled and excluded zones.
plugin: amazon.aws.aws_ec2
regions:
  - us-east-1
route53_enabled: true
route53_excluded_zones:
  - ansible.bar.net
  - ansible.foo.net
"""

import re

try:
    import botocore
except ImportError:
    pass  # will be captured by imported HAS_BOTO3

from collections import defaultdict
from typing import Any
from typing import Dict
from typing import List
from typing import Set

from ansible.module_utils.common.text.converters import to_text

try:
    from ansible.template import trust_as_template
except ImportError:
    trust_as_template = None
from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.transformation import ansible_dict_to_boto3_filter_list
from ansible_collections.amazon.aws.plugins.plugin_utils.inventory import AWSInventoryBase

# The mappings give an array of keys to get from the filter name to the value
# returned by boto3's EC2 describe_instances method.

instance_meta_filter_to_boto_attr = {
    "group-id": ("Groups", "GroupId"),
    "group-name": ("Groups", "GroupName"),
    "network-interface.attachment.instance-owner-id": ("OwnerId",),
    "owner-id": ("OwnerId",),
    "requester-id": ("RequesterId",),
    "reservation-id": ("ReservationId",),
}

instance_data_filter_to_boto_attr = {
    "affinity": ("Placement", "Affinity"),
    "architecture": ("Architecture",),
    "availability-zone": ("Placement", "AvailabilityZone"),
    "block-device-mapping.attach-time": ("BlockDeviceMappings", "Ebs", "AttachTime"),
    "block-device-mapping.delete-on-termination": ("BlockDeviceMappings", "Ebs", "DeleteOnTermination"),
    "block-device-mapping.device-name": ("BlockDeviceMappings", "DeviceName"),
    "block-device-mapping.status": ("BlockDeviceMappings", "Ebs", "Status"),
    "block-device-mapping.volume-id": ("BlockDeviceMappings", "Ebs", "VolumeId"),
    "client-token": ("ClientToken",),
    "dns-name": ("PublicDnsName",),
    "host-id": ("Placement", "HostId"),
    "hypervisor": ("Hypervisor",),
    "iam-instance-profile.arn": ("IamInstanceProfile", "Arn"),
    "image-id": ("ImageId",),
    "instance-id": ("InstanceId",),
    "instance-lifecycle": ("InstanceLifecycle",),
    "instance-state-code": ("State", "Code"),
    "instance-state-name": ("State", "Name"),
    "instance-type": ("InstanceType",),
    "instance.group-id": ("SecurityGroups", "GroupId"),
    "instance.group-name": ("SecurityGroups", "GroupName"),
    "ip-address": ("PublicIpAddress",),
    "kernel-id": ("KernelId",),
    "key-name": ("KeyName",),
    "launch-index": ("AmiLaunchIndex",),
    "launch-time": ("LaunchTime",),
    "monitoring-state": ("Monitoring", "State"),
    "network-interface.addresses.private-ip-address": ("NetworkInterfaces", "PrivateIpAddress"),
    "network-interface.addresses.primary": ("NetworkInterfaces", "PrivateIpAddresses", "Primary"),
    "network-interface.addresses.association.public-ip": (
        "NetworkInterfaces",
        "PrivateIpAddresses",
        "Association",
        "PublicIp",
    ),
    "network-interface.addresses.association.ip-owner-id": (
        "NetworkInterfaces",
        "PrivateIpAddresses",
        "Association",
        "IpOwnerId",
    ),
    "network-interface.association.public-ip": ("NetworkInterfaces", "Association", "PublicIp"),
    "network-interface.association.ip-owner-id": ("NetworkInterfaces", "Association", "IpOwnerId"),
    "network-interface.association.allocation-id": ("ElasticGpuAssociations", "ElasticGpuId"),
    "network-interface.association.association-id": ("ElasticGpuAssociations", "ElasticGpuAssociationId"),
    "network-interface.attachment.attachment-id": ("NetworkInterfaces", "Attachment", "AttachmentId"),
    "network-interface.attachment.instance-id": ("InstanceId",),
    "network-interface.attachment.device-index": ("NetworkInterfaces", "Attachment", "DeviceIndex"),
    "network-interface.attachment.status": ("NetworkInterfaces", "Attachment", "Status"),
    "network-interface.attachment.attach-time": ("NetworkInterfaces", "Attachment", "AttachTime"),
    "network-interface.attachment.delete-on-termination": ("NetworkInterfaces", "Attachment", "DeleteOnTermination"),
    "network-interface.availability-zone": ("Placement", "AvailabilityZone"),
    "network-interface.description": ("NetworkInterfaces", "Description"),
    "network-interface.group-id": ("NetworkInterfaces", "Groups", "GroupId"),
    "network-interface.group-name": ("NetworkInterfaces", "Groups", "GroupName"),
    "network-interface.ipv6-addresses.ipv6-address": ("NetworkInterfaces", "Ipv6Addresses", "Ipv6Address"),
    "network-interface.mac-address": ("NetworkInterfaces", "MacAddress"),
    "network-interface.network-interface-id": ("NetworkInterfaces", "NetworkInterfaceId"),
    "network-interface.owner-id": ("NetworkInterfaces", "OwnerId"),
    "network-interface.private-dns-name": ("NetworkInterfaces", "PrivateDnsName"),
    # 'network-interface.requester-id': (),
    "network-interface.requester-managed": ("NetworkInterfaces", "Association", "IpOwnerId"),
    "network-interface.status": ("NetworkInterfaces", "Status"),
    "network-interface.source-dest-check": ("NetworkInterfaces", "SourceDestCheck"),
    "network-interface.subnet-id": ("NetworkInterfaces", "SubnetId"),
    "network-interface.vpc-id": ("NetworkInterfaces", "VpcId"),
    "placement-group-name": ("Placement", "GroupName"),
    "platform": ("Platform",),
    "private-dns-name": ("PrivateDnsName",),
    "private-ip-address": ("PrivateIpAddress",),
    "product-code": ("ProductCodes", "ProductCodeId"),
    "product-code.type": ("ProductCodes", "ProductCodeType"),
    "ramdisk-id": ("RamdiskId",),
    "reason": ("StateTransitionReason",),
    "root-device-name": ("RootDeviceName",),
    "root-device-type": ("RootDeviceType",),
    "source-dest-check": ("SourceDestCheck",),
    "spot-instance-request-id": ("SpotInstanceRequestId",),
    "state-reason-code": ("StateReason", "Code"),
    "state-reason-message": ("StateReason", "Message"),
    "subnet-id": ("SubnetId",),
    "tag": ("Tags",),
    "tag-key": ("Tags",),
    "tag-value": ("Tags",),
    "tenancy": ("Placement", "Tenancy"),
    "virtualization-type": ("VirtualizationType",),
    "vpc-id": ("VpcId",),
}


def _get_tag_hostname(preference, instance):
    tag_hostnames = preference.split("tag:", 1)[1]
    expected_single_value = False
    if "," in tag_hostnames:
        tag_hostnames = tag_hostnames.split(",")
    else:
        expected_single_value = True
        tag_hostnames = [tag_hostnames]

    tags = boto3_tag_list_to_ansible_dict(instance.get("Tags", []))
    tag_values = []
    for v in tag_hostnames:
        if "=" in v:
            tag_name, tag_value = v.split("=")
            if tags.get(tag_name) == tag_value:
                tag_values.append(to_text(tag_name) + "_" + to_text(tag_value))
        else:
            tag_value = tags.get(v)
            if tag_value:
                tag_values.append(to_text(tag_value))
    if expected_single_value and len(tag_values) > 0:
        tag_values = tag_values[0]
    return tag_values


def _prepare_host_vars(
    original_host_vars,
    hostvars_prefix=None,
    hostvars_suffix=None,
    use_contrib_script_compatible_ec2_tag_keys=False,
):
    host_vars = camel_dict_to_snake_dict(original_host_vars, ignore_list=["Tags"])
    host_vars["ec2_tags"] = boto3_tag_list_to_ansible_dict(original_host_vars.get("Tags", []))
    # ec2_tags is the new key, tags is deprecated but kept for backward compatibility
    host_vars["tags"] = host_vars["ec2_tags"]

    # Allow easier grouping by region
    host_vars["placement"]["region"] = host_vars["placement"]["availability_zone"][:-1]

    if use_contrib_script_compatible_ec2_tag_keys:
        for k, v in host_vars["ec2_tags"].items():
            host_vars[f"ec2_tag_{k}"] = v

    if hostvars_prefix or hostvars_suffix:
        for hostvar, hostval in host_vars.copy().items():
            del host_vars[hostvar]
            if hostvars_prefix:
                hostvar = hostvars_prefix + hostvar
            if hostvars_suffix:
                hostvar = hostvar + hostvars_suffix
            host_vars[hostvar] = hostval

    return host_vars


def _compile_values(obj, attr):
    """
    :param obj: A list or dict of instance attributes
    :param attr: A key
    :return The value(s) found via the attr
    """
    if obj is None:
        return

    temp_obj = []

    if isinstance(obj, list) or isinstance(obj, tuple):
        for each in obj:
            value = _compile_values(each, attr)
            if value:
                temp_obj.append(value)
    else:
        temp_obj = obj.get(attr)

    has_indexes = any([isinstance(temp_obj, list), isinstance(temp_obj, tuple)])
    if has_indexes and len(temp_obj) == 1:
        return temp_obj[0]

    return temp_obj


def _get_boto_attr_chain(filter_name, instance):
    """
    :param filter_name: The filter
    :param instance: instance dict returned by boto3 ec2 describe_instances()
    """
    allowed_filters = sorted(
        list(instance_data_filter_to_boto_attr.keys()) + list(instance_meta_filter_to_boto_attr.keys())
    )

    # If filter not in allow_filters -> use it as a literal string
    if filter_name not in allowed_filters:
        return filter_name

    if filter_name in instance_data_filter_to_boto_attr:
        boto_attr_list = instance_data_filter_to_boto_attr[filter_name]
    else:
        boto_attr_list = instance_meta_filter_to_boto_attr[filter_name]

    instance_value = instance
    for attribute in boto_attr_list:
        instance_value = _compile_values(instance_value, attribute)
    return instance_value


def _describe_ec2_instances(connection, filters):
    paginator = connection.get_paginator("describe_instances")
    return paginator.paginate(Filters=filters).build_full_result()


def _get_ssm_information(client, filters):
    paginator = client.get_paginator("get_inventory")
    return paginator.paginate(Filters=filters).build_full_result()


@AWSRetry.jittered_backoff()
def _list_hosted_zones(client: Any, **kwargs: Dict[str, Any]) -> List[Dict[str, Any]]:
    """
    List all hosted zones using the given boto3 Route53 client.

    :param client: boto3 client for Route53
    :param kwargs: Additional parameters for the paginator
    :return: List of hosted zones
    """
    paginator = client.get_paginator("list_hosted_zones")
    return paginator.paginate(**kwargs).build_full_result()["HostedZones"]


@AWSRetry.jittered_backoff()
def _list_resource_record_sets(client: Any, hosted_zone_id: str, **kwargs: Dict[str, Any]) -> List[Dict[str, Any]]:
    """
    Retrieve all resource record sets for a specific hosted zone in Route53.

    :param client: A boto3 Route53 client.
    :param hosted_zone_id: The ID of the hosted zone whose records should be listed.
    :param kwargs: Optional keyword arguments to pass to the paginator.
    :return: A list of dictionaries representing resource record sets.
    """
    paginator = client.get_paginator("list_resource_record_sets")
    return paginator.paginate(HostedZoneId=hosted_zone_id, **kwargs).build_full_result()["ResourceRecordSets"]


def _remove_trailing_dot(data: str) -> str:
    """Remove the trailing dot from a string"""
    if data.endswith("."):
        return data[:-1]
    return data


class InventoryModule(AWSInventoryBase):
    NAME = "amazon.aws.aws_ec2"
    INVENTORY_FILE_SUFFIXES = ("aws_ec2.yml", "aws_ec2.yaml")

    def __init__(self):
        super().__init__()

        self.group_prefix = "aws_ec2_"

    def _get_instances_by_region(self, regions, filters, strict_permissions):
        """
        :param regions: a list of regions in which to describe instances
        :param filters: a list of boto3 filter dictionaries
        :param strict_permissions: a boolean determining whether to fail or ignore 403 error codes
        :return A list of instance dictionaries
        """
        all_instances = []
        # By default find non-terminated/terminating instances
        if not any(f["Name"] == "instance-state-name" for f in filters):
            filters.append({"Name": "instance-state-name", "Values": ["running", "pending", "stopping", "stopped"]})

        for connection, _region in self.all_clients("ec2"):
            try:
                reservations = _describe_ec2_instances(connection, filters).get("Reservations")
                instances = []
                for r in reservations:
                    new_instances = r["Instances"]
                    reservation_details = {
                        "OwnerId": r["OwnerId"],
                        "RequesterId": r.get("RequesterId", ""),
                        "ReservationId": r["ReservationId"],
                    }
                    for instance in new_instances:
                        instance.update(reservation_details)
                    instances.extend(new_instances)
            except is_boto3_error_code("UnauthorizedOperation") as e:
                if not strict_permissions:
                    continue
                self.fail_aws("Failed to describe instances", exception=e)
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                self.fail_aws("Failed to describe instances", exception=e)

            all_instances.extend(instances)

        return all_instances

    def _sanitize_hostname(self, hostname):
        if ":" in to_text(hostname):
            return self._sanitize_group_name(to_text(hostname))
        else:
            return to_text(hostname)

    def _get_hostname_with_jinja2_filter(self, instance, preference, return_single_hostname=False):
        jinja2_filter = None
        is_template = False
        if "|" in preference:
            preference, jinja2_filter = preference.split("|", maxsplit=1)
            preference = preference.rstrip()
            is_template = True
        if preference.startswith("tag:"):
            hostname = _get_tag_hostname(preference, instance)
        else:
            hostname = _get_boto_attr_chain(preference, instance)
        if is_template:
            template_var = "{{'%s'|%s}}" % (hostname, jinja2_filter)
            if isinstance(hostname, list):
                template_var = "{{%s|%s}}" % (hostname, jinja2_filter)
            if trust_as_template:
                template_var = trust_as_template(template_var)
            hostname = self.templar.template(variable=template_var)
        if isinstance(hostname, list) and return_single_hostname:
            hostname = hostname[0] if hostname else None
        return hostname

    @property
    def route53_enabled(self) -> bool:
        return self.get_option("route53_enabled")

    def _get_preferred_hostname(self, instance, hostnames):
        """
        :param instance: an instance dict returned by boto3 ec2 describe_instances()
        :param hostnames: a list of hostname destination variables in order of preference
        :return the preferred identifer for the host
        """
        if self.route53_enabled:
            route53_hostnames = self._get_instance_route53_hostnames(instance)
            if route53_hostnames:
                # This method returns only one hostname
                return self._sanitize_hostname(sorted(route53_hostnames)[0])

        if not hostnames:
            hostnames = ["dns-name", "private-dns-name"]

        hostname = None
        for preference in hostnames:
            if isinstance(preference, dict):
                if "name" not in preference:
                    self.fail_aws("A 'name' key must be defined in a hostnames dictionary.")
                hostname = self._get_preferred_hostname(instance, [preference["name"]])
                hostname_from_prefix = None
                if "prefix" in preference:
                    hostname_from_prefix = self._get_preferred_hostname(instance, [preference["prefix"]])
                separator = preference.get("separator", "_")
                if hostname and hostname_from_prefix and "prefix" in preference:
                    hostname = hostname_from_prefix + separator + hostname
            else:
                hostname = self._get_hostname_with_jinja2_filter(instance, preference, return_single_hostname=True)
            if hostname:
                break
        if hostname:
            return self._sanitize_hostname(hostname)

    def _get_all_hostnames(self, instance, hostnames):
        """
        :param instance: an instance dict returned by boto3 ec2 describe_instances()
        :param hostnames: a list of hostname destination variables
        :return all the candidats matching the expectation
        """
        hostname_list = []
        if self.route53_enabled:
            hostname_list = [self._sanitize_hostname(name) for name in self._get_instance_route53_hostnames(instance)]

        if not hostnames:
            hostnames = ["dns-name", "private-dns-name"]

        hostname = None
        for preference in hostnames:
            if isinstance(preference, dict):
                if "name" not in preference:
                    self.fail_aws("A 'name' key must be defined in a hostnames dictionary.")
                hostname = self._get_all_hostnames(instance, [preference["name"]])
                hostname_from_prefix = None
                if "prefix" in preference:
                    hostname_from_prefix = self._get_all_hostnames(instance, [preference["prefix"]])
                separator = preference.get("separator", "_")
                if hostname and hostname_from_prefix and "prefix" in preference:
                    hostname = hostname_from_prefix[0] + separator + hostname[0]
            else:
                hostname = self._get_hostname_with_jinja2_filter(instance, preference)

            if hostname:
                if isinstance(hostname, list):
                    for host in hostname:
                        hostname_list.append(self._sanitize_hostname(host))
                elif isinstance(hostname, str):
                    hostname_list.append(self._sanitize_hostname(hostname))

        return hostname_list

    def _query(self, regions, include_filters, exclude_filters, strict_permissions, use_ssm_inventory):
        """
        :param regions: a list of regions to query
        :param include_filters: a list of boto3 filter dictionaries
        :param exclude_filters: a list of boto3 filter dictionaries
        :param strict_permissions: a boolean determining whether to fail or ignore 403 error codes

        """
        instances = []
        ids_to_ignore = []
        for filter_dict in exclude_filters:
            for i in self._get_instances_by_region(
                regions,
                ansible_dict_to_boto3_filter_list(filter_dict),
                strict_permissions,
            ):
                ids_to_ignore.append(i["InstanceId"])
        for filter_dict in include_filters:
            for i in self._get_instances_by_region(
                regions,
                ansible_dict_to_boto3_filter_list(filter_dict),
                strict_permissions,
            ):
                if i["InstanceId"] not in ids_to_ignore:
                    instances.append(i)
                    ids_to_ignore.append(i["InstanceId"])

        instances = sorted(instances, key=lambda x: x["InstanceId"])

        if use_ssm_inventory and instances:
            for connection, _region in self.all_clients("ssm"):
                self._add_ssm_information(connection, instances)

        return {"aws_ec2": instances}

    def _add_ssm_information(self, connection, instances):
        instance_ids = [x["InstanceId"] for x in instances]
        result = self._get_multiple_ssm_inventories(connection, instance_ids)
        for entity in result.get("Entities", []):
            for x in instances:
                if x["InstanceId"] == entity["Id"]:
                    content = entity.get("Data", {}).get("AWS:InstanceInformation", {}).get("Content", [])
                    if content:
                        x["SsmInventory"] = content[0]
                    break

    def _map_route53_records(self) -> Dict[str, Set[str]]:
        """Get and store the map of resource records to domain names that point to them."""
        route53_excluded_zones = [_remove_trailing_dot(zone) for zone in self.get_option("route53_excluded_zones")]
        route53_map = defaultdict(set)
        for connection, _region in self.all_clients("route53"):
            for zone in _list_hosted_zones(connection):
                if _remove_trailing_dot(zone["Name"]) in route53_excluded_zones:
                    continue
                for record in _list_resource_record_sets(client=connection, hosted_zone_id=zone["Id"]):
                    for resource in record.get("ResourceRecords", []):
                        route53_map[_remove_trailing_dot(resource["Value"])].add(_remove_trailing_dot(record["Name"]))
        return route53_map

    def _is_matching_route53_hostname(self, hostname: str) -> bool:
        route53_hostnames = self.get_option("route53_hostnames")
        result = True
        if route53_hostnames:
            result = any((hostname.endswith(name) for name in route53_hostnames))
        return result

    def _get_instance_route53_hostnames(self, instance: Dict[str, Any]) -> List[str]:
        """Check if an instance is referenced in the records we have from
        Route53. If it is, return the list of domain names pointing to that
        instance. If nothing points to it, return an empty list."""

        hostnames = set()
        for attr in ("PublicDnsName", "PrivateDnsName", "PublicIpAddress", "PrivateIpAddress"):
            if (attr_value := instance.get(attr)) and attr_value in self.route53_resource_record_mapping:
                hostnames.update(self.route53_resource_record_mapping[attr_value])
        return [name for name in hostnames if self._is_matching_route53_hostname(name)]

    def _get_multiple_ssm_inventories(self, connection, instance_ids):
        result = []
        # SSM inventory filters Values list can contain a maximum of 40 items so we need to retrieve 40 at a time
        # https://docs.aws.amazon.com/systems-manager/latest/APIReference/API_InventoryFilter.html
        while len(instance_ids) > 40:
            filters = [{"Key": "AWS:InstanceInformation.InstanceId", "Values": instance_ids[:40]}]
            result.extend(_get_ssm_information(connection, filters).get("Entities", []))
            instance_ids = instance_ids[40:]
        if instance_ids:
            filters = [{"Key": "AWS:InstanceInformation.InstanceId", "Values": instance_ids}]
            result.extend(_get_ssm_information(connection, filters).get("Entities", []))
        return {"Entities": result}

    def _populate(
        self,
        groups,
        hostnames,
        allow_duplicated_hosts=False,
        hostvars_prefix=None,
        hostvars_suffix=None,
        use_contrib_script_compatible_ec2_tag_keys=False,
    ):
        for group in groups:
            group = self.inventory.add_group(group)
            self._add_hosts(
                hosts=groups[group],
                group=group,
                hostnames=hostnames,
                allow_duplicated_hosts=allow_duplicated_hosts,
                hostvars_prefix=hostvars_prefix,
                hostvars_suffix=hostvars_suffix,
                use_contrib_script_compatible_ec2_tag_keys=use_contrib_script_compatible_ec2_tag_keys,
            )
            self.inventory.add_child("all", group)

    def iter_entry(
        self,
        hosts,
        hostnames,
        allow_duplicated_hosts=False,
        hostvars_prefix=None,
        hostvars_suffix=None,
        use_contrib_script_compatible_ec2_tag_keys=False,
    ):
        for host in hosts:
            if allow_duplicated_hosts:
                hostname_list = self._get_all_hostnames(host, hostnames)
            else:
                hostname_list = [self._get_preferred_hostname(host, hostnames)]
            if not hostname_list or hostname_list[0] is None:
                continue

            host_vars = _prepare_host_vars(
                host,
                hostvars_prefix,
                hostvars_suffix,
                use_contrib_script_compatible_ec2_tag_keys,
            )
            for name in hostname_list:
                yield to_text(name), host_vars

    def _add_hosts(
        self,
        hosts,
        group,
        hostnames,
        allow_duplicated_hosts=False,
        hostvars_prefix=None,
        hostvars_suffix=None,
        use_contrib_script_compatible_ec2_tag_keys=False,
    ):
        """
        :param hosts: a list of hosts to be added to a group
        :param group: the name of the group to which the hosts belong
        :param hostnames: a list of hostname destination variables in order of preference
        :param bool allow_duplicated_hosts: if true, accept same host with different names
        :param str hostvars_prefix: starts the hostvars variable name with this prefix
        :param str hostvars_suffix: ends the hostvars variable name with this suffix
        :param bool use_contrib_script_compatible_ec2_tag_keys: transform the host name with the legacy naming system
        """

        for name, host_vars in self.iter_entry(
            hosts,
            hostnames,
            allow_duplicated_hosts=allow_duplicated_hosts,
            hostvars_prefix=hostvars_prefix,
            hostvars_suffix=hostvars_suffix,
            use_contrib_script_compatible_ec2_tag_keys=use_contrib_script_compatible_ec2_tag_keys,
        ):
            self.inventory.add_host(name, group=group)
            for k, v in host_vars.items():
                self.inventory.set_variable(name, k, v)

            # Use constructed if applicable

            strict = self.get_option("strict")

            # Composed variables
            self._set_composite_vars(self.get_option("compose"), host_vars, name, strict=strict)

            # Complex groups based on jinja2 conditionals, hosts that meet the conditional are added to group
            self._add_host_to_composed_groups(self.get_option("groups"), host_vars, name, strict=strict)

            # Create groups based on variable values and add the corresponding hosts to it
            self._add_host_to_keyed_groups(self.get_option("keyed_groups"), host_vars, name, strict=strict)

    def build_include_filters(self):
        result = self.get_option("include_filters")
        if self.get_option("filters"):
            result = [self.get_option("filters")] + result
        return result or [{}]

    def parse(self, inventory, loader, path, cache=True):
        super().parse(inventory, loader, path, cache=cache)

        self.display.deprecated(
            "The 'tags' host variable is deprecated. Use 'ec2_tags' instead.",
            date="2026-12-01",
            collection_name="amazon.aws",
        )

        # get user specifications
        regions = self.get_option("regions")
        include_filters = self.build_include_filters()
        exclude_filters = self.get_option("exclude_filters")
        hostnames = self.get_option("hostnames")
        strict_permissions = self.get_option("strict_permissions")
        allow_duplicated_hosts = self.get_option("allow_duplicated_hosts")

        hostvars_prefix = self.get_option("hostvars_prefix")
        hostvars_suffix = self.get_option("hostvars_suffix")
        use_contrib_script_compatible_sanitization = self.get_option("use_contrib_script_compatible_sanitization")
        use_contrib_script_compatible_ec2_tag_keys = self.get_option("use_contrib_script_compatible_ec2_tag_keys")
        use_ssm_inventory = self.get_option("use_ssm_inventory")

        if use_contrib_script_compatible_sanitization:
            self.display.deprecated(
                "The 'use_contrib_script_compatible_sanitization' option is deprecated. "
                "Use Ansible's default group name sanitization instead.",
                date="2026-12-01",
                collection_name="amazon.aws",
            )

            self._sanitize_group_name = self._legacy_script_compatible_group_sanitization

        if use_contrib_script_compatible_ec2_tag_keys:
            self.display.deprecated(
                "The 'use_contrib_script_compatible_ec2_tag_keys' option is deprecated. "
                "Use the 'ec2_tags' structure instead.",
                date="2026-12-01",
                collection_name="amazon.aws",
            )

        if not all(isinstance(element, (dict, str)) for element in hostnames):
            self.fail_aws("Hostnames should be a list of dict and str.")

        result_was_cached, results = self.get_cached_result(path, cache)

        if not result_was_cached:
            results = self._query(regions, include_filters, exclude_filters, strict_permissions, use_ssm_inventory)

        if self.route53_enabled:
            self.route53_resource_record_mapping = self._map_route53_records()

        self._populate(
            results,
            hostnames,
            allow_duplicated_hosts=allow_duplicated_hosts,
            hostvars_prefix=hostvars_prefix,
            hostvars_suffix=hostvars_suffix,
            use_contrib_script_compatible_ec2_tag_keys=use_contrib_script_compatible_ec2_tag_keys,
        )

        self.update_cached_result(path, cache, results)

    @staticmethod
    def _legacy_script_compatible_group_sanitization(name):
        # note that while this mirrors what the script used to do, it has many issues with unicode and usability in python
        regex = re.compile(r"[^A-Za-z0-9\_\-]")

        return regex.sub("_", name)



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_monitordatacollectionruleassociation_info
version_added: "3.9.0"
short_description: Get or list Data Collection Rule Association
description:
    - Get or list Data Collection Rule Association.

options:
    data_collection_endpoint_name:
        description:
            - The name of the data collection endpoint.
            - The name is case insensitive.
        type: str
    data_collection_rule_name:
        description:
            - The name of the data collection rule.
            - The name is case insensitive.
        type: str
    resource_uri:
        description:
            - The identifier of the resource.
        type: str
    association_name:
        description:
            - The name of the association.
            - The name is case insensitive.
        type: str
    resource_group:
        description:
            - The name of the resource group in which the data collection rule association is (if you use name)
        type: str
extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - magodo (@magodo)
    - Fred Sun (@Fred-sun)
'''

EXAMPLES = '''
- name: Get data collection rule association details
  azure.azcollection.azure_rm_monitordatacollectionrulesassociation_info:
    association_name: association01
    resource_uri: "/subscriptions/xxx-xxx/resourceGroups/v-xisuRG/providers/Microsoft.Compute/virtualMachines/fredVM"

- name: List all data collection rule associations with data_collection_endpoint_name
  azure.azcollection.azure_rm_monitordatacollectionrulesassociation_info:
    resource_group: Resource_Group_Name
    data_collection_endpoint_name: fredrpfx001-DCE

- name: List all data collection rule associations with data_collection_rule_name
  azure.azcollection.azure_rm_monitordatacollectionrulesassociation_info:
    resource_group: Resource_Group_Name
    data_collection_rule_name: fredrpfx001-DCR

- name: List all data collection rule associations with the resource_uri
  azure.azcollection.azure_rm_monitordatacollectionrulesassociation_info:
    resource_uri: "/subscriptions/xxx-xxx/resourceGroups/v-xisuRG/providers/Microsoft.Compute/virtualMachines/fredVM"
'''

RETURN = '''
datacollectionruleassociations:
    description:
        - List of data collection rule association.
        - Can be empty if listing data collection rule association.
    type: complex
    returned: always
    contains:
        data_collection_rule_id:
            description:
                - The resource ID of the data collection endpoint that is to be associated.
            type: str
            returned: when-used
            sample: "/subscriptions/xxxxx/resourceGroups/v-xisuRG02/providers/Microsoft.Insights/dataCollectionRules/fredrpfx001-DCR"
        data_collection_endpoint_id:
            description:
                - The resource ID of the data collection endpoint that is to be associated.
            type: str
            returned: when-used
            sample: "/subscriptions/xxx-xxx/resourceGroups/v-xisuRG/providers/Microsoft.Insights/dataCollectionEndpoints/fredendpoint"
        description:
            description:
                - Description of the association.
            type: str
            returned: always
            sample: Create
        etag:
            description:
                - Description of the association.
            returned: always
            type: str
            sample: "0c00c2b5-0000-0800-0000-68ca0de80000"
        id:
            description:
                - Fully qualified ID of the resource.
            type: str
            returned: always
            sample: "/subscriptions/xxxxxxxxxx/resourceGroups/v-xisuRG/providers/Microsoft.Compute/virtualMachines/\
                     fredVM/providers/Microsoft.Insights/dataCollectionRuleAssociations/association01"
        name:
            description:
                - Description of the association.
            type: str
            returned: always
            sample: associationname
        system_data:
            description:
                - Metadata pertaining to creation and last modification of the resource.
            type: dict
            returned: always
            sample: {
                "created_at": "2025-09-17T01:24:56.450551Z",
                "created_by": "00867800-0fa3-4d02-8bc8-35edac3a0d32",
                "created_by_type": "Application",
                "last_modified_at": "2025-09-17T01:24:56.450551Z",
                "last_modified_by": "00867800-0fa3-4d02-8bc8-35edac3a0d32",
                "last_modified_by_type": "Application"
            }
        type:
            description:
                - The type of the resource.
            type: str
            returned: always
            sample: "Microsoft.Insights/dataCollectionRuleAssociations"
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase


class AzureRMDataCollectionRuleAssociationInfo(AzureRMModuleBase):
    """Information class for an Azure RM Data Collection Rules"""

    def __init__(self):
        self.module_arg_spec = dict(
            resource_uri=dict(type='str'),
            association_name=dict(type='str'),
            data_collection_endpoint_name=dict(type='str'),
            data_collection_rule_name=dict(type='str'),
            resource_group=dict(type='str')
        )

        self.required_by = {
            'data_collection_rule_name': 'resource_group',
            'data_collection_endpoint_name': 'resource_group',
            'association_name': 'resource_uri'
        }

        self.resource_uri = None
        self.resource_group = None
        self.association_name = None
        self.data_collection_endpoint_name = None
        self.data_collection_rule_name = None
        self.log_path = None
        self.log_mode = None

        self.results = dict(
            changed=False,
            datacollectionruleassociations=[]
        )

        super(AzureRMDataCollectionRuleAssociationInfo, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                                       supports_check_mode=True,
                                                                       supports_tags=False,
                                                                       facts_module=True,
                                                                       required_by=self.required_by)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        result = []
        if self.association_name:
            result = self.get_association(self.resource_uri, self.association_name)
        elif self.data_collection_rule_name:
            result = self.list_by_rule()
        elif self.data_collection_endpoint_name:
            result = self.list_by_data_collection_endpoint()
        elif self.resource_uri:
            result = self.list_by_resource()
        else:
            self.fail("One of association_name, data_collection_rule_name, data_collection_endpoint_name and resource_uri must be configured")

        self.results['datacollectionruleassociations'] = result

        return self.results

    def get_association(self, resource_uri, association_name):
        '''
        Gets the specified association
        '''
        result = []
        response = None

        try:
            response = self.monitor_management_client_data_collection_rules.data_collection_rule_associations.get(resource_uri=resource_uri,
                                                                                                                  association_name=association_name)
        except Exception:
            self.log("Could not find data collection rule assoication {0} in resource uri {1}".format(self.association_name, self.resource_uri))
            return []
        if response:
            result = [response.as_dict()]

        return result

    def list_by_resource(self):
        '''
        Lists associations for the specified resource.
        '''
        result = []
        response = None

        try:
            response = self.monitor_management_client_data_collection_rules.data_collection_rule_associations.list_by_resource(resource_uri=self.resource_uri)
        except Exception as ex:
            self.log("Could not list data collection rule assoication in resource uri {0}, Exception as {1}".format(self.resource_uri, ex))
            return []
        if response:
            for item in response:
                result.append(item.as_dict())

        return result

    def list_by_rule(self):
        '''
        Lists associations for the specified data collection rule.
        '''
        result = []
        response = None
        try:
            response = self.monitor_management_client_data_collection_rules.data_collection_rule_associations.list_by_rule(
                resource_group_name=self.resource_group,
                data_collection_rule_name=self.data_collection_rule_name)
        except Exception as ex:
            self.log("Could not list assoication in data collection rule {0}, Exception as {1}".format(self.data_collection_rule_name, ex))
            return []
        if response:
            for item in response:
                association = item.as_dict()
                result += self.get_association(association['id'].split('providers/microsoft.insights')[0], association['name'])

        return result

    def list_by_data_collection_endpoint(self):
        '''
        Lists associations for the specified data collection endpoint.
        '''
        result = []
        response = None

        try:
            response = self.monitor_management_client_data_collection_rules.data_collection_rule_associations.list_by_data_collection_endpoint(
                resource_group_name=self.resource_group,
                data_collection_endpoint_name=self.data_collection_endpoint_name)
        except Exception as ex:
            self.log("Could not list associations for the data collection rule endpoint {0}, Exception as {1}".format(self.data_collection_endpoint_name, ex))
            return []
        if response:
            for item in response:
                result.append(item.as_dict())

        return result


def main():
    """Main execution"""
    AzureRMDataCollectionRuleAssociationInfo()


if __name__ == '__main__':
    main()



# This code is part of the Ansible collection community.docker, but is an independent component.
# This particular file, and this file only, is based on the Docker SDK for Python (https://github.com/docker/docker-py/)
#
# Copyright (c) 2016-2022 Docker, Inc.
#
# It is licensed under the Apache 2.0 license (see LICENSES/Apache-2.0.txt in this collection)
# SPDX-License-Identifier: Apache-2.0

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import typing as t

from ansible.module_utils.common.text.converters import to_text

from ._import_helper import HTTPError as _HTTPError

if t.TYPE_CHECKING:
    from requests import Response


class DockerException(Exception):
    """
    A base class from which all other exceptions inherit.

    If you want to catch all errors that the Docker SDK might raise,
    catch this base exception.
    """


def create_api_error_from_http_exception(e: _HTTPError) -> t.NoReturn:
    """
    Create a suitable APIError from requests.exceptions.HTTPError.
    """
    response = e.response
    try:
        explanation = response.json()["message"]
    except ValueError:
        explanation = to_text((response.content or "").strip())
    cls = APIError
    if response.status_code == 404:
        if explanation and (
            "No such image" in str(explanation)
            or "not found: does not exist or no pull access" in str(explanation)
            or "repository does not exist" in str(explanation)
        ):
            cls = ImageNotFound
        else:
            cls = NotFound
    raise cls(e, response=response, explanation=explanation) from e


class APIError(_HTTPError, DockerException):
    """
    An HTTP error from the API.
    """

    def __init__(
        self,
        message: str | Exception,
        response: Response | None = None,
        explanation: str | None = None,
    ) -> None:
        # requests 1.2 supports response as a keyword argument, but
        # requests 1.1 does not
        super().__init__(message)
        self.response = response
        self.explanation = explanation or ""

    def __str__(self) -> str:
        message = super().__str__()

        if self.is_client_error():
            message = f"{self.response.status_code} Client Error for {self.response.url}: {self.response.reason}"

        elif self.is_server_error():
            message = f"{self.response.status_code} Server Error for {self.response.url}: {self.response.reason}"

        if self.explanation:
            message = f'{message} ("{self.explanation}")'

        return message

    @property
    def status_code(self) -> int | None:
        if self.response is not None:
            return self.response.status_code
        return None

    def is_error(self) -> bool:
        return self.is_client_error() or self.is_server_error()

    def is_client_error(self) -> bool:
        if self.status_code is None:
            return False
        return 400 <= self.status_code < 500

    def is_server_error(self) -> bool:
        if self.status_code is None:
            return False
        return 500 <= self.status_code < 600


class NotFound(APIError):
    pass


class ImageNotFound(NotFound):
    pass


class InvalidVersion(DockerException):
    pass


class InvalidRepository(DockerException):
    pass


class InvalidConfigFile(DockerException):
    pass


class InvalidArgument(DockerException):
    pass


class DeprecatedMethod(DockerException):
    pass


class TLSParameterError(DockerException):
    def __init__(self, msg: str) -> None:
        self.msg = msg

    def __str__(self) -> str:
        return self.msg + (
            ". TLS configurations should map the Docker CLI "
            "client configurations. See "
            "https://docs.docker.com/engine/articles/https/ "
            "for API details."
        )


class NullResource(DockerException, ValueError):
    pass


class ContainerError(DockerException):
    """
    Represents a container that has exited with a non-zero exit code.
    """

    def __init__(
        self,
        container: str,
        exit_status: int,
        command: list[str],
        image: str,
        stderr: str | None,
    ):
        self.container = container
        self.exit_status = exit_status
        self.command = command
        self.image = image
        self.stderr = stderr

        err = f": {stderr}" if stderr is not None else ""
        msg = f"Command '{command}' in image '{image}' returned non-zero exit status {exit_status}{err}"

        super().__init__(msg)


class StreamParseError(RuntimeError):
    def __init__(self, reason: Exception) -> None:
        self.msg = reason


class BuildError(DockerException):
    def __init__(self, reason: str, build_log: str) -> None:
        super().__init__(reason)
        self.msg = reason
        self.build_log = build_log


class ImageLoadError(DockerException):
    pass


def create_unexpected_kwargs_error(name: str, kwargs: dict[str, t.Any]) -> TypeError:
    quoted_kwargs = [f"'{k}'" for k in sorted(kwargs)]
    text = [f"{name}() "]
    if len(quoted_kwargs) == 1:
        text.append("got an unexpected keyword argument ")
    else:
        text.append("got unexpected keyword arguments ")
    text.append(", ".join(quoted_kwargs))
    return TypeError("".join(text))


class MissingContextParameter(DockerException):
    def __init__(self, param: str) -> None:
        self.param = param

    def __str__(self) -> str:
        return f"missing parameter: {self.param}"


class ContextAlreadyExists(DockerException):
    def __init__(self, name: str) -> None:
        self.name = name

    def __str__(self) -> str:
        return f"context {self.name} already exists"


class ContextException(DockerException):
    def __init__(self, msg: str) -> None:
        self.msg = msg

    def __str__(self) -> str:
        return self.msg


class ContextNotFound(DockerException):
    def __init__(self, name: str) -> None:
        self.name = name

    def __str__(self) -> str:
        return f"context '{self.name}' not found"


class MissingRequirementException(DockerException):
    def __init__(
        self, msg: str, requirement: str, import_exception: ImportError | str
    ) -> None:
        self.msg = msg
        self.requirement = requirement
        self.import_exception = import_exception

    def __str__(self) -> str:
        return self.msg



#!/usr/bin/python
# Copyright (c) 2022, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: nameserver_record_info
short_description: Look up all records of a type from all nameservers for a DNS name
version_added: 2.6.0
description:
  - Given a DNS name and a record type, will retrieve all nameservers that are responsible for this DNS name, and from them
    all records for this name of the given type.
extends_documentation_fragment:
  - community.dns._attributes
  - community.dns._attributes.info_module
  - community.dns._attributes.idempotent_not_modify_state
author:
  - Felix Fontein (@felixfontein)
options:
  name:
    description:
      - A list of DNS names whose nameservers to retrieve.
    required: true
    type: list
    elements: str
  type:
    description:
      - The record type to retrieve.
      - Support for V(HTTPS) and V(SVCB) has been added in community.dns 3.4.0.
    required: true
    type: str
    choices:
      - A
      - AAAA
      - CAA
      - CNAME
      - DNAME
      - DNSKEY
      - DS
      - HINFO
      - HTTPS
      - LOC
      - MX
      - NAPTR
      - NS
      - NSEC
      - NSEC3
      - NSEC3PARAM
      - PTR
      - RP
      - RRSIG
      - SOA
      - SPF
      - SRV
      - SSHFP
      - SVCB
      - TLSA
      - TXT
  query_retry:
    description:
      - Number of retries for DNS query timeouts.
    type: int
    default: 3
  query_timeout:
    description:
      - Timeout per DNS query in seconds.
    type: float
    default: 10
  always_ask_default_resolver:
    description:
      - When set to V(true) (default), will use the default resolver to find the authoritative nameservers of a subzone. See
        O(server) for how to configure the default resolver.
      - When set to V(false), will use the authoritative nameservers of the parent zone to find the authoritative nameservers
        of a subzone. This only makes sense when the nameservers were recently changed and have not yet propagated.
    type: bool
    default: true
  servfail_retries:
    description:
      - How often to retry on SERVFAIL errors.
    type: int
    default: 0
  server:
    description:
      - The DNS server(s) to use to look up the result. Must be a list of one or more IP addresses.
      - By default, the system's standard resolver is used.
    type: list
    elements: str
    version_added: 2.7.0
requirements:
  - dnspython >= 2.0.0
"""

EXAMPLES = r"""
- name: Retrieve TXT values from all nameservers for two DNS names
  community.dns.nameserver_record_info:
    name:
      - www.example.com
      - example.org
    type: TXT
  register: result

- name: Show TXT values for www.example.com for all nameservers
  ansible.builtin.debug:
    msg: '{{ result.results[0].result }}'
"""

RETURN = r"""
results:
  description:
    - Information on the records for every DNS name provided in O(name).
  returned: always
  type: list
  elements: dict
  contains:
    name:
      description:
        - The DNS name this entry is for.
      returned: always
      type: str
      sample: www.example.com
    result:
      description:
        - A list of values per nameserver.
      returned: success
      type: list
      elements: dict
      sample:
        - nameserver: ns1.example.com
          entries:
            - X
        - nameserver: ns2.example.com
          entries:
            - X
      contains:
        nameserver:
          description:
            - The nameserver.
          returned: success
          type: str
          sample: ns1.example.com
        values:
          description:
            - The records of type O(type).
            - Depending on O(type), different fields are returned.
            - For O(type=TXT) and O(type=SPF), also the concatenated value is returned as RV(results[].result[].entries[].value).
            - B(The field has been renamed) to RV(results[].result[].entries) in community.dns 3.4.0.
              While the old name will be around for a longer time, prefer using the new one.
          returned: success
          type: list
          elements: dict
        entries:
          description:
            - The records of type O(type).
            - Depending on O(type), different fields are returned.
            - For O(type=TXT) and O(type=SPF), also the concatenated value is returned as RV(results[].result[].entries[].value).
            - This field has been called RV(results[].result[].values) before.
          version_added: 3.4.0
          returned: success
          type: list
          elements: dict
          sample:
            - address: 127.0.0.1
          contains:
            address:
              description:
                - A IPv4 respectively IPv6 address.
              type: str
              returned: if O(type=A) or O(type=AAAA)
            algorithm:
              description:
                - The algorithm ID.
              type: int
              returned: if O(type=DNSKEY) or O(type=DS) or O(type=NSEC3) or O(type=NSEC3PARAM) or O(type=RRSIG) or O(type=SSHFP)
            altitude:
              description:
                - The altitude.
              type: float
              returned: if O(type=LOC)
            cert:
              description:
                - The certificate.
              type: str
              returned: if O(type=TLSA)
            cpu:
              description:
                - The CPU.
              type: str
              returned: if O(type=HINFO)
            digest:
              description:
                - The digest.
              type: str
              returned: if O(type=DS)
            digest_type:
              description:
                - The digest's type.
              type: int
              returned: if O(type=DS)
            exchange:
              description:
                - The exchange server.
              type: str
              returned: if O(type=MX)
            expiration:
              description:
                - The expiration Unix timestamp.
              type: int
              returned: if O(type=RRSIG)
            expire:
              description:
                - Number of seconds after which secondary name servers should stop answering request for this zone if the
                  main name server does not respond.
              type: int
              returned: if O(type=SOA)
            fingerprint:
              description:
                - The fingerprint.
              type: str
              returned: if O(type=SSHFP)
            flags:
              description:
                - Flags.
                - This is actually of type C(string) for O(type=NAPTR).
              type: int
              returned: if O(type=CAA) or O(type=DNSKEY) or O(type=NAPTR) or O(type=NSEC3) or O(type=NSEC3PARAM)
            fp_type:
              description:
                - The fingerprint's type.
              type: int
              returned: if O(type=SSHFP)
            horizontal_precision:
              description:
                - The horizontal precision of the location.
              type: float
              returned: if O(type=LOC)
            inception:
              description:
                - The inception Unix timestamp.
              type: int
              returned: if O(type=RRSIG)
            iterations:
              description:
                - The number of iterations.
              type: int
              returned: if O(type=NSEC3) or O(type=NSEC3PARAM)
            key:
              description:
                - The key.
              type: str
              returned: if O(type=DNSKEY)
            key_tag:
              description:
                - The key's tag.
              type: int
              returned: if O(type=DS) or O(type=RRSIG)
            labels:
              description:
                - The labels.
              type: int
              returned: if O(type=RRSIG)
            latitude:
              description:
                - The location's latitude.
              type: list
              elements: int
              returned: if O(type=LOC)
            longitude:
              description:
                - The location's longitude.
              type: list
              elements: int
              returned: if O(type=LOC)
            mbox:
              description:
                - The mbox.
              type: str
              returned: if O(type=RP)
            minimum:
              description:
                - Used to calculate the TTL for purposes of negative caching.
              type: int
              returned: if O(type=SOA)
            mname:
              description:
                - Primary main name server for this zone.
              type: str
              returned: if O(type=SOA)
            mtype:
              description:
                - The mtype.
              type: int
              returned: if O(type=TLSA)
            next:
              description:
                - The next value.
              type: str
              returned: if O(type=NSEC) or O(type=NSEC3)
            order:
              description:
                - The order value.
              type: int
              returned: if O(type=NAPTR)
            original_ttl:
              description:
                - The original TTL.
              type: int
              returned: if O(type=RRSIG)
            os:
              description:
                - The operating system.
              type: str
              returned: if O(type=HINFO)
            params:
              description:
                - The parameters. See L(RFC 9460, https://datatracker.ietf.org/doc/rfc9460/?include_text=1) for details.
              type: dict
              returned: if O(type=HTTPS) or O(type=SVCB)
              contains:
                mandatory:
                  description:
                    - Keys of parameters that are considered mandatory.
                  type: list
                  elements: str
                alpn:
                  description:
                    - List of Base64-encoded ALPN IDs.
                  type: list
                  elements: str
                no-default-alpn:
                  description:
                    - The value will always be V(null) if this key is present.
                  type: raw
                port:
                  description:
                    - A port.
                  type: int
                ipv4hint:
                  description:
                    - A list of IPv4 addresses.
                  type: list
                  elements: str
                ech:
                  description:
                    - A Base64-encoded ECH (Encrypted Client Hello) key.
                  type: raw
                ipv6hint:
                  description:
                    - A list of IPv6 addresses.
                  type: list
                  elements: str
                ohttp:
                  description:
                    - The value will always be V(null) if this key is present.
                  type: raw
            port:
              description:
                - The port.
              type: int
              returned: if O(type=SRV)
            preference:
              description:
                - The preference value for this record.
              type: int
              returned: if O(type=MX) or O(type=NAPTR)
            priority:
              description:
                - The priority value for this record.
              type: int
              returned: if O(type=HTTPS), O(type=SRV), or O(type=SVCB)
            protocol:
              description:
                - The protocol.
              type: int
              returned: if O(type=DNSKEY)
            refresh:
              description:
                - Number of seconds after which secondary name servers should query the main name server for the SOA record
                  to detect zone changes.
              type: int
              returned: if O(type=SOA)
            regexp:
              description:
                - A regular expression.
              type: str
              returned: if O(type=NAPTR)
            replacement:
              description:
                - The replacement.
              type: str
              returned: if O(type=NAPTR)
            retry:
              description:
                - Number of seconds after which secondary name servers should retry to request the serial number from the
                  main name server if the main name server does not respond.
              type: int
              returned: if O(type=SOA)
            rname:
              description:
                - E-mail address of the administrator responsible for this zone.
              type: str
              returned: if O(type=SOA)
            salt:
              description:
                - The salt.
              type: str
              returned: if O(type=NSEC3) or O(type=NSEC3PARAM)
            selector:
              description:
                - The selector.
              type: int
              returned: if O(type=TLSA)
            serial:
              description:
                - Serial number for this zone.
              type: int
              returned: if O(type=SOA)
            service:
              description:
                - The service.
              type: str
              returned: if O(type=NAPTR)
            signature:
              description:
                - The signature.
              type: str
              returned: if O(type=RRSIG)
            signer:
              description:
                - The signer.
              type: str
              returned: if O(type=RRSIG)
            size:
              description:
                - The size of the location.
              type: float
              returned: if O(type=LOC)
            strings:
              description:
                - List of strings for this record.
                - See RV(results[].result[].entries[].value) for the concatenated result.
              type: list
              elements: str
              returned: if O(type=SPF) or O(type=TXT)
            tag:
              description:
                - The tag.
              type: str
              returned: if O(type=CAA)
            target:
              description:
                - The target.
              type: str
              returned: if O(type=CNAME), O(type=DNAME), O(type=HTTPS), O(type=NS), O(type=PTR), O(type=SRV), or O(type=SVCB)
            txt:
              description:
                - The TXT value.
              type: str
              returned: if O(type=RP)
            type_covered:
              description:
                - The type covered.
              type: str
              returned: if O(type=RRSIG)
            usage:
              description:
                - The usage flag.
              type: int
              returned: if O(type=TLSA)
            value:
              description:
                - The value.
                - For O(type=SPF) or O(type=TXT), this is the concatenation of RV(results[].result[].entries[].strings).
              type: str
              returned: if O(type=CAA) or O(type=SPF) or O(type=TXT)
            vertical_precision:
              description:
                - The vertical precision of the location.
              type: float
              returned: if O(type=LOC)
            weight:
              description:
                - The service's weight.
              type: int
              returned: if O(type=SRV)
            windows:
              description:
                - The windows.
              type: str
              returned: if O(type=NSEC) or O(type=NSEC3)
  sample:
    - name: www.example.com
      result:
        - nameserver: ns1.example.com
          values:
            - address: 127.0.0.1
        - nameserver: ns2.example.com
          values:
            - address: 127.0.0.1
    - name: example.org
      result:
        - nameserver: ns1.example.org
          values:
            - address: 127.0.0.1
            - address: 127.0.0.2
        - nameserver: ns2.example.org
          values:
            - address: 127.0.0.2
        - nameserver: ns3.example.org
          values:
            - address: 127.0.0.1
"""

import typing as t

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.dns.plugins.module_utils._dnspython_records import (
    NAME_TO_RDTYPE,
    NAME_TO_REQUIRED_VERSION,
    convert_rdata_to_dict,
)
from ansible_collections.community.dns.plugins.module_utils._resolver import (
    ResolveDirectlyFromNameServers,
    assert_requirements_present,
    guarded_run,
)


def main() -> None:
    module = AnsibleModule(
        argument_spec={
            "name": {"required": True, "type": "list", "elements": "str"},
            "type": {
                "required": True,
                "type": "str",
                "choices": [
                    "A",
                    "AAAA",
                    "CAA",
                    "CNAME",
                    "DNAME",
                    "DNSKEY",
                    "DS",
                    "HINFO",
                    "HTTPS",
                    "LOC",
                    "MX",
                    "NAPTR",
                    "NS",
                    "NSEC",
                    "NSEC3",
                    "NSEC3PARAM",
                    "PTR",
                    "RP",
                    "RRSIG",
                    "SOA",
                    "SPF",
                    "SRV",
                    "SSHFP",
                    "SVCB",
                    "TLSA",
                    "TXT",
                ],
            },
            "query_retry": {"type": "int", "default": 3},
            "query_timeout": {"type": "float", "default": 10},
            "always_ask_default_resolver": {"type": "bool", "default": True},
            "servfail_retries": {"type": "int", "default": 0},
            "server": {"type": "list", "elements": "str"},
        },
        supports_check_mode=True,
    )
    assert_requirements_present(module)

    names = module.params["name"]
    record_type = module.params["type"]

    resolver = ResolveDirectlyFromNameServers(
        timeout=module.params["query_timeout"],
        timeout_retries=module.params["query_retry"],
        servfail_retries=module.params["servfail_retries"],
        always_ask_default_resolver=module.params["always_ask_default_resolver"],
        server_addresses=module.params["server"],
    )
    results: list[dict[str, t.Any]] = [{"name": name} for name in names]

    if record_type not in NAME_TO_RDTYPE:
        min_version = NAME_TO_REQUIRED_VERSION[record_type]
        module.fail_json(
            msg=f"Your dnspython version does not support {record_type} records. You need version {min_version} or newer."
        )
    rdtype = NAME_TO_RDTYPE[record_type]

    def f():
        for index, name in enumerate(names):
            result = []
            results[index]["result"] = result
            records_for_nameservers = resolver.resolve(name, rdtype=rdtype)
            for nameserver, records in records_for_nameservers.items():
                ns_result = {
                    "nameserver": nameserver,
                }
                result.append(ns_result)
                values = []
                if records is not None:
                    for data in records:
                        values.append(convert_rdata_to_dict(data))
                ns_result["values"] = values
                ns_result["entries"] = values
            result.sort(key=lambda v: v["nameserver"])

    guarded_run(f, module, generate_additional_results=lambda: {"results": results})
    module.exit_json(results=results)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: nxos_reboot
extends_documentation_fragment:
- cisco.nxos.nxos
short_description: Reboot a network device.
description:
- Reboot a network device.
version_added: 1.0.0
author:
- Jason Edelman (@jedelman8)
- Gabriele Gerbino (@GGabriele)
notes:
- Tested against NXOSv 7.3.(0)D1(1) on VIRL
- Tested against Cisco MDS NX-OS 9.2(1)
- The module will fail due to timeout issues, but the reboot will be performed anyway.
options:
  confirm:
    description:
    - Safeguard boolean. Set to true if you're sure you want to reboot.
    required: false
    default: false
    type: bool
"""

EXAMPLES = """
- cisco.nxos.nxos_reboot:
    confirm: true
"""

RETURN = """
rebooted:
    description: Whether the device was instructed to reboot.
    returned: success
    type: bool
    sample: true
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import load_config


def reboot(module):
    cmds = "terminal dont-ask ; reload"
    opts = {"ignore_timeout": True}
    load_config(module, cmds, False, opts)


def main():
    argument_spec = dict(confirm=dict(default=False, type="bool"))

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)

    warnings = list()
    results = dict(changed=False, warnings=warnings)

    if module.params["confirm"]:
        if not module.check_mode:
            reboot(module)
        results["changed"] = True

    module.exit_json(**results)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright 2023 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


"""
The fact_diff filter plugin
"""
from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
    name: fact_diff
    author: Ashwini Mhatre ((@amhatre))
    version_added: 2.12.0
    short_description: Find the difference between currently set facts
    description:
      - Compare two facts or variables and get a diff.
    options:
      before:
        description:
          - The first fact to be used in the comparison.
        type: raw
        required: true
      after:
        description:
          - The second fact to be used in the comparison.
        type: raw
        required: true
      plugin:
        description:
          - Configure and specify the diff plugin to use
        type: dict
        default: {}
        suboptions:
          name:
            description:
              - 'The diff plugin to use, in fully qualified collection name format.'
            default: ansible.utils.native
            type: str
          vars:
            description:
              - Parameters passed to the diff plugin.
            type: dict
            default: {}
            suboptions:
              skip_lines:
                description:
                  - Skip lines matching these regular expressions.
                  - Matches will be removed prior to the diff.
                  - >-
                    If the provided I(before) and I(after) are a string, they will
                    be split.
                  - >-
                    Each entry in each list will be cast to a string for the
                    comparison
                type: list
                elements: str
      common:
        description:
          - Show all common lines.
        type: bool
        default: false
"""

EXAMPLES = """
- name: Set fact
  ansible.builtin.set_fact:
    before:
      a:
        b:
          c:
            d:
              - 0
              - 1
    after:
      a:
        b:
          c:
            d:
              - 2
              - 3

- name: Show the difference in json format
  ansible.builtin.set_fact:
    result: "{{before | ansible.utils.fact_diff(after)}}"

# TASK [Show the difference in json format] **********************************************************************************************
# ok: [localhost] => {
#     "ansible_facts": {
#         "result": [
#             "--- before",
#             "+++ after",
#             "@@ -3,8 +3,8 @@",
#             "         \"b\": {",
#             "             \"c\": {",
#             "                 \"d\": [",
#             "-                    0,",
#             "-                    1",
#             "+                    2,",
#             "+                    3",
#             "                 ]",
#             "             }",
#             "         }",
#             ""
#         ]
#     },
#     "changed": false
# }

- name: Set fact
  ansible.builtin.set_fact:
    before: "{{ before|ansible.utils.to_paths }}"
    after: "{{ after|ansible.utils.to_paths }}"

- name: Show the difference in path format
  ansible.builtin.set_fact:
    result: "{{before | ansible.utils.fact_diff(after)}}"

# TASK [Show the difference in path format] **********************************************************************************************
# ok: [localhost] => {
#     "ansible_facts": {
#         "result": [
#             "--- before",
#             "+++ after",
#             "@@ -1,4 +1,4 @@",
#             " {",
#             "-    \"a.b.c.d[0]\": 0,",
#             "-    \"a.b.c.d[1]\": 1",
#             "+    \"a.b.c.d[0]\": 2,",
#             "+    \"a.b.c.d[1]\": 3",
#             " }",
#             ""
#         ]
#     },
#     "changed": false
# }

- name: Set fact
  ansible.builtin.set_fact:
    before: "{{ before|to_nice_yaml }}"
    after: "{{ after|to_nice_yaml }}"

- name: Show the difference in yaml format
  ansible.builtin.set_fact:
    result: "{{before | ansible.utils.fact_diff(after)}}"

# TASK [Show the difference in yaml format] **********************************************************************************************
# ok: [localhost] => {
#     "ansible_facts": {
#         "result": [
#             "--- before",
#             "+++ after",
#             "@@ -1,2 +1,2 @@",
#             "-a.b.c.d[0]: 0",
#             "-a.b.c.d[1]: 1",
#             "+a.b.c.d[0]: 2",
#             "+a.b.c.d[1]: 3",
#             ""
#         ]
#     },
#     "changed": false
# }
"""
RETURN = """
  result:
    type: list
    description:
      - Returns diff between before and after facts.
"""
from ansible.errors import AnsibleFilterError
from ansible.module_utils.common.text.converters import to_text

from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
    AnsibleArgSpecValidator,
)
from ansible_collections.ansible.utils.plugins.plugin_utils.fact_diff import fact_diff


try:
    from jinja2.filters import pass_environment
except ImportError:
    from jinja2.filters import environmentfilter as pass_environment


@pass_environment
def _fact_diff(*args, **kwargs):
    """Find the difference between currently set facts"""

    keys = ["before", "after", "plugin", "common"]
    data = dict(zip(keys, args[1:]))
    data.update(kwargs)
    aav = AnsibleArgSpecValidator(data=data, schema=DOCUMENTATION, name="fact_diff")
    valid, errors, updated_data = aav.validate()
    if not valid:
        raise AnsibleFilterError(errors)
    res = fact_diff(**updated_data)
    return to_text(res)


class FilterModule(object):
    """fact_diff"""

    def filters(self):
        """a mapping of filter names to functions"""
        return {"fact_diff": _fact_diff}



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_wireless_devices_provisioning_deployments
short_description: Resource module for organizations _wireless _devices _provisioning
  _deployments
description:
  - Manage operations create, update and delete of the resource organizations _wireless
    _devices _provisioning _deployments.
  - Create a zero touch deployment for a wireless access point.
  - Delete a zero touch deployment.
  - Update a zero touch deployment.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  deploymentId:
    description: DeploymentId path parameter. Deployment ID.
    type: str
  items:
    description: List of zero touch deployments to create.
    elements: dict
    suboptions:
      completedAt:
        description: Timestamp of when the zero touch deployment request was completed.
        type: str
      createdAt:
        description: Timestamp of when the zero touch deployment request was created.
        type: str
      deploymentId:
        description: Zero touch deployment request identifier.
        type: str
      devices:
        description: An array composed of old and new devices.
        suboptions:
          new:
            description: New device.
            suboptions:
              mac:
                description: MAC address of the new device.
                type: str
              model:
                description: Model of the new device.
                type: str
              name:
                description: Name of the new device or serial number if not named.
                type: str
              rfProfile:
                description: RF profile of the new device.
                suboptions:
                  id:
                    description: ID of RfProfile for new device.
                    type: str
                  name:
                    description: Name of RfProfile for new device.
                    type: str
                type: dict
              serial:
                description: Serial number of the new device.
                type: str
              tags:
                description: Tag(s) of the new device.
                elements: str
                type: list
            type: dict
          old:
            description: Old device.
            suboptions:
              afterAction:
                description: Action to be taken on the old device.
                type: str
              mac:
                description: MAC address of the old device.
                type: str
              model:
                description: Model of the old device.
                type: str
              name:
                description: Name of the old device.
                type: str
              rfProfile:
                description: RF profile of the old device.
                suboptions:
                  id:
                    description: ID of the RF profile.
                    type: str
                  name:
                    description: Name of the RF profile.
                    type: str
                type: dict
              serial:
                description: Serial number of the old device.
                type: str
              tags:
                description: Tag(s) of the old device.
                elements: str
                type: list
            type: dict
        type: dict
      errors:
        description: Array of error message(s) if any.
        elements: str
        type: list
      lastUpdatedAt:
        description: Timestamp of when the zero touch deployment request was last
          updated.
        type: str
      network:
        description: Network information.
        suboptions:
          id:
            description: ID of the network the device is being added to.
            type: str
          name:
            description: Name of the network the device is being added to.
            type: str
        type: dict
      requestedAt:
        description: Timestamp of when the zero touch deployment request was created.
        type: str
      status:
        description: Status of the zero touch deployment request. Enum = ready, in
          progress, completed, failed.
        type: str
      type:
        description: Type of the zero touch deployment request. Enum = deploy, replace.
        type: str
    type: list
  meta:
    description: Metadata relevant to the paginated dataset.
    suboptions:
      counts:
        description: Counts relating to the paginated dataset.
        suboptions:
          items:
            description: Counts relating to the paginated items.
            suboptions:
              remaining:
                description: The number of items in the dataset that are available
                  on subsequent pages.
                type: int
              total:
                description: The total number of items in the dataset.
                type: int
            type: dict
        type: dict
    type: dict
  organizationId:
    description: OrganizationId path parameter. Organization ID.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for wireless createOrganizationWirelessDevicesProvisioningDeployment
    description: Complete reference of the createOrganizationWirelessDevicesProvisioningDeployment
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-organization-wireless-devices-provisioning-deployment
  - name: Cisco Meraki documentation for wireless deleteOrganizationWirelessDevicesProvisioningDeployment
    description: Complete reference of the deleteOrganizationWirelessDevicesProvisioningDeployment
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!delete-organization-wireless-devices-provisioning-deployment
  - name: Cisco Meraki documentation for wireless updateOrganizationWirelessDevicesProvisioningDeployments
    description: Complete reference of the updateOrganizationWirelessDevicesProvisioningDeployments
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-organization-wireless-devices-provisioning-deployments
notes:
  - SDK Method used are
    wireless.Wireless.create_organization_wireless_devices_provisioning_deployment,
    wireless.Wireless.delete_organization_wireless_devices_provisioning_deployment,
    wireless.Wireless.update_organization_wireless_devices_provisioning_deployments,
  - Paths used are
    post /organizations/{organizationId}/wireless/devices/provisioning/deployments,
    delete /organizations/{organizationId}/wireless/devices/provisioning/deployments/{deploymentId},
    put /organizations/{organizationId}/wireless/devices/provisioning/deployments,
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.organizations_wireless_devices_provisioning_deployments:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    items:
      - completedAt: '2018-02-11T00:00:00.090210Z'
        createdAt: '2018-02-11T00:00:00.090210Z'
        deploymentId: '1284392014819'
        devices:
          new:
            mac: 00:11:22:33:44:55
            model: CW9166I
            name: My AP
            rfProfile:
              id: '1284392014819'
              name: RF Profile Name
            serial: Q234-ABCD-5678
            tags:
              - tag1
              - tag2
          old:
            afterAction: unclaim
            mac: 00:11:22:33:44:55
            model: MR34
            name: My AP
            rfProfile:
              id: '1284392014819'
              name: RF Profile Name
            serial: Q234-ABCD-5678
            tags:
              - tag1
              - tag2
        errors:
          - error message1
          - error message2
        lastUpdatedAt: '2018-02-11T00:00:00.090210Z'
        network:
          id: N_24329156
          name: Main Office
        requestedAt: '2018-02-11T00:00:00.090210Z'
        status: ready
        type: replace
    meta:
      counts:
        items:
          remaining: 0
          total: 20
    organizationId: string
- name: Update all
  cisco.meraki.organizations_wireless_devices_provisioning_deployments:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    items:
      - completedAt: '2018-02-11T00:00:00.090210Z'
        createdAt: '2018-02-11T00:00:00.090210Z'
        deploymentId: '1284392014819'
        devices:
          new:
            mac: 00:11:22:33:44:55
            model: CW9166I
            name: My AP
            rfProfile:
              id: '1284392014819'
              name: RF Profile Name
            serial: Q234-ABCD-5678
            tags:
              - tag1
              - tag2
          old:
            afterAction: unclaim
            mac: 00:11:22:33:44:55
            model: MR34
            name: My AP
            rfProfile:
              id: '1284392014819'
              name: RF Profile Name
            serial: Q234-ABCD-5678
            tags:
              - tag1
              - tag2
        errors:
          - error message1
          - error message2
        lastUpdatedAt: '2018-02-11T00:00:00.090210Z'
        network:
          id: N_24329156
          name: Main Office
        requestedAt: '2018-02-11T00:00:00.090210Z'
        status: ready
        type: replace
    meta:
      counts:
        items:
          remaining: 0
          total: 20
    organizationId: string
- name: Delete by id
  cisco.meraki.organizations_wireless_devices_provisioning_deployments:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: absent
    deploymentId: string
    organizationId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "items": [
        {
          "deploymentId": "string",
          "devices": {
            "old": {
              "serial": "string",
              "afterAction": "string",
              "name": "string",
              "model": "string",
              "mac": "string",
              "tags": [
                "string"
              ],
              "rfProfile": {
                "id": "string",
                "name": "string"
              }
            },
            "new": {
              "serial": "string",
              "name": "string",
              "model": "string",
              "mac": "string",
              "tags": [
                "string"
              ],
              "rfProfile": {
                "id": "string",
                "name": "string"
              }
            }
          },
          "status": "string",
          "type": "string",
          "network": {
            "id": "string",
            "name": "string"
          },
          "createdAt": "string",
          "requestedAt": "string",
          "lastUpdatedAt": "string",
          "completedAt": "string",
          "errors": [
            "string"
          ]
        }
      ],
      "meta": {
        "counts": {
          "items": {
            "total": 0,
            "remaining": 0
          }
        }
      }
    }
"""



#!/usr/bin/python
#
# Copyright (c) 2015, Steve Gargan <steve.gargan@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: consul
short_description: Add, modify & delete services within a Consul cluster
description:
  - Registers services and checks for an agent with a Consul cluster. A service is some process running on the agent node
    that should be advertised by Consul's discovery mechanism. It may optionally supply a check definition, a periodic service
    test to notify the Consul cluster of service's health.
  - Checks may also be registered per node, for example disk usage, or cpu usage and notify the health of the entire node
    to the cluster. Service level checks do not require a check name or ID as these are derived by Consul from the Service
    name and ID respectively by appending V(service:) Node level checks require a O(check_name) and optionally a O(check_id).
  - Currently, there is no complete way to retrieve the script, interval or TTL metadata for a registered check. Without this
    metadata it is not possible to tell if the data supplied with ansible represents a change to a check. As a result this
    does not attempt to determine changes and it always reports a changed occurred. An API method is planned to supply this
    metadata so at that stage change management is to be added.
  - See U(http://consul.io) for more details.
requirements:
  - py-consul
  - requests
author: "Steve Gargan (@sgargan)"
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
options:
  state:
    type: str
    description:
      - Register or deregister the Consul service, defaults to present.
    default: present
    choices: ['present', 'absent']
  service_name:
    type: str
    description:
      - Unique name for the service on a node, must be unique per node, required if registering a service. May be omitted
        if registering a node level check.
  service_id:
    type: str
    description:
      - The ID for the service, must be unique per node. If O(state=absent), defaults to the service name if supplied.
  host:
    type: str
    description:
      - Host of the Consul agent defaults to localhost.
    default: localhost
  port:
    type: int
    description:
      - The port on which the Consul agent is running.
    default: 8500
  scheme:
    type: str
    description:
      - The protocol scheme on which the Consul agent is running.
    default: http
  validate_certs:
    description:
      - Whether to verify the TLS certificate of the Consul agent.
    type: bool
    default: true
  notes:
    type: str
    description:
      - Notes to attach to check when registering it.
  service_port:
    type: int
    description:
      - The port on which the service is listening. Can optionally be supplied for registration of a service, that is if O(service_name)
        or O(service_id) is set.
  service_address:
    type: str
    description:
      - The address to advertise that the service is listening on. This value is passed as the C(address) parameter to Consul's
        C(/v1/agent/service/register) API method, so refer to the Consul API documentation for further details.
  tags:
    type: list
    elements: str
    description:
      - Tags that are attached to the service registration.
  script:
    type: str
    description:
      - The script/command that is run periodically to check the health of the service.
      - Requires O(interval) to be provided.
      - Mutually exclusive with O(ttl), O(tcp) and O(http).
  interval:
    type: str
    description:
      - The interval at which the service check is run. This is a number with a V(s) or V(m) suffix to signify the units of
        seconds or minutes, for example V(15s) or V(1m). If no suffix is supplied V(s) is used by default, for example V(10)
        is V(10s).
      - Required if one of the parameters O(script), O(http), or O(tcp) is specified.
  check_id:
    type: str
    description:
      - An ID for the service check. If O(state=absent), defaults to O(check_name). Ignored if part of a service definition.
  check_name:
    type: str
    description:
      - Name for the service check. Required if standalone, ignored if part of service definition.
  check_node:
    description:
      - Node name.
    type: str
  check_host:
    description:
      - Host name.
    type: str
  ttl:
    type: str
    description:
      - Checks can be registered with a TTL instead of a O(script) and O(interval) this means that the service checks in with
        the agent before the TTL expires. If it does not the check is considered failed. Required if registering a check and
        the script an interval are missing Similar to the interval this is a number with a V(s) or V(m) suffix to signify
        the units of seconds or minutes, for example V(15s) or V(1m). If no suffix is supplied V(s) is used by default, for
        example V(10) is equivalent to V(10s).
      - Mutually exclusive with O(script), O(tcp) and O(http).
  tcp:
    type: str
    description:
      - Checks can be registered with a TCP port. This means that Consul checks if the connection attempt to that port is
        successful (that is, the port is currently accepting connections). The format is V(host:port), for example V(localhost:80).
      - Requires O(interval) to be provided.
      - Mutually exclusive with O(script), O(ttl) and O(http).
    version_added: '1.3.0'
  http:
    type: str
    description:
      - Checks can be registered with an HTTP endpoint. This means that Consul checks that the http endpoint returns a successful
        HTTP status.
      - Requires O(interval) to be provided.
      - Mutually exclusive with O(script), O(ttl) and O(tcp).
  timeout:
    type: str
    description:
      - A custom HTTP check timeout. The Consul default is 10 seconds. Similar to the interval this is a number with a V(s)
        or V(m) suffix to signify the units of seconds or minutes, for example V(15s) or V(1m). If no suffix is supplied V(s)
        is used by default, for example V(10) is equivalent to V(10s).
  token:
    type: str
    description:
      - The token key identifying an ACL rule set. May be required to register services.
"""

EXAMPLES = r"""
- name: Register nginx service with the local Consul agent
  community.general.consul:
    service_name: nginx
    service_port: 80

- name: Register nginx service with curl check
  community.general.consul:
    service_name: nginx
    service_port: 80
    script: curl http://localhost
    interval: 60s

- name: register nginx with a tcp check
  community.general.consul:
    service_name: nginx
    service_port: 80
    interval: 60s
    tcp: localhost:80

- name: Register nginx with an http check
  community.general.consul:
    service_name: nginx
    service_port: 80
    interval: 60s
    http: http://localhost:80/status

- name: Register external service nginx available at 10.1.5.23
  community.general.consul:
    service_name: nginx
    service_port: 80
    service_address: 10.1.5.23

- name: Register nginx with some service tags
  community.general.consul:
    service_name: nginx
    service_port: 80
    tags:
      - prod
      - webservers

- name: Remove nginx service
  community.general.consul:
    service_name: nginx
    state: absent

- name: Register celery worker service
  community.general.consul:
    service_name: celery-worker
    tags:
      - prod
      - worker

- name: Create a node level check to test disk usage
  community.general.consul:
    check_name: Disk usage
    check_id: disk_usage
    script: /opt/disk_usage.py
    interval: 5m

- name: Register an http check against a service that's already registered
  community.general.consul:
    check_name: nginx-check2
    check_id: nginx-check2
    service_id: nginx
    interval: 60s
    http: http://localhost:80/morestatus
"""

try:
    import consul
    from requests.exceptions import ConnectionError

    class PatchedConsulAgentService(consul.Consul.Agent.Service):
        def deregister(self, service_id, token=None):
            params = {}
            if token:
                params["token"] = token
            return self.agent.http.put(
                consul.base.CB.bool(), f"/v1/agent/service/deregister/{service_id}", params=params
            )

    python_consul_installed = True
except ImportError:
    python_consul_installed = False

import re

from ansible.module_utils.basic import AnsibleModule


def register_with_consul(module):
    state = module.params["state"]

    if state == "present":
        add(module)
    else:
        remove(module)


def add(module):
    """adds a service or a check depending on supplied configuration"""
    check = parse_check(module)
    service = parse_service(module)

    if not service and not check:
        module.fail_json(msg="a name and port are required to register a service")

    if service:
        if check:
            service.add_check(check)
        add_service(module, service)
    elif check:
        add_check(module, check)


def remove(module):
    """removes a service or a check"""
    service_id = module.params["service_id"] or module.params["service_name"]
    check_id = module.params["check_id"] or module.params["check_name"]
    if service_id:
        remove_service(module, service_id)
    else:
        remove_check(module, check_id)


def add_check(module, check):
    """registers a check with the given agent. currently there is no way
    retrieve the full metadata of an existing check  through the consul api.
    Without this we can't compare to the supplied check and so we must assume
    a change."""
    if not check.name and not check.service_id:
        module.fail_json(msg="a check name is required for a node level check, one not attached to a service")

    consul_api = get_consul_api(module)
    check.register(consul_api)

    module.exit_json(
        changed=True,
        check_id=check.check_id,
        check_name=check.name,
        script=check.script,
        interval=check.interval,
        ttl=check.ttl,
        tcp=check.tcp,
        http=check.http,
        timeout=check.timeout,
        service_id=check.service_id,
    )


def remove_check(module, check_id):
    """removes a check using its id"""
    consul_api = get_consul_api(module)

    if check_id in consul_api.agent.checks():
        consul_api.agent.check.deregister(check_id)
        module.exit_json(changed=True, id=check_id)

    module.exit_json(changed=False, id=check_id)


def add_service(module, service):
    """registers a service with the current agent"""
    result = service
    changed = False

    consul_api = get_consul_api(module)
    existing = get_service_by_id_or_name(consul_api, service.id)

    # there is no way to retrieve the details of checks so if a check is present
    # in the service it must be re-registered
    if service.has_checks() or not existing or existing != service:
        service.register(consul_api)
        # check that it registered correctly
        registered = get_service_by_id_or_name(consul_api, service.id)
        if registered:
            result = registered
            changed = True

    module.exit_json(
        changed=changed,
        service_id=result.id,
        service_name=result.name,
        service_port=result.port,
        checks=[check.to_dict() for check in service.checks()],
        tags=result.tags,
    )


def remove_service(module, service_id):
    """deregister a service from the given agent using its service id"""
    consul_api = get_consul_api(module)
    service = get_service_by_id_or_name(consul_api, service_id)
    if service:
        consul_api.agent.service.deregister(service_id, token=module.params["token"])
        module.exit_json(changed=True, id=service_id)

    module.exit_json(changed=False, id=service_id)


def get_consul_api(module):
    consulClient = consul.Consul(
        host=module.params["host"],
        port=module.params["port"],
        scheme=module.params["scheme"],
        verify=module.params["validate_certs"],
        token=module.params["token"],
    )
    consulClient.agent.service = PatchedConsulAgentService(consulClient)
    return consulClient


def get_service_by_id_or_name(consul_api, service_id_or_name):
    """iterate the registered services and find one with the given id"""
    for dummy, service in consul_api.agent.services().items():
        if service_id_or_name in (service["ID"], service["Service"]):
            return ConsulService(loaded=service)


def parse_check(module):
    if module.params["check_id"] or any(module.params[p] is not None for p in ("script", "ttl", "tcp", "http")):
        return ConsulCheck(
            module.params["check_id"],
            module.params["check_name"],
            module.params["check_node"],
            module.params["check_host"],
            module.params["script"],
            module.params["interval"],
            module.params["ttl"],
            module.params["notes"],
            module.params["tcp"],
            module.params["http"],
            module.params["timeout"],
            module.params["service_id"],
        )


def parse_service(module):
    return ConsulService(
        module.params["service_id"],
        module.params["service_name"],
        module.params["service_address"],
        module.params["service_port"],
        module.params["tags"],
    )


class ConsulService:
    def __init__(self, service_id=None, name=None, address=None, port=-1, tags=None, loaded=None):
        self.id = self.name = name
        if service_id:
            self.id = service_id
        self.address = address
        self.port = port
        self.tags = tags
        self._checks = []
        if loaded:
            self.id = loaded["ID"]
            self.name = loaded["Service"]
            self.port = loaded["Port"]
            self.tags = loaded["Tags"]

    def register(self, consul_api):
        optional = {}

        if self.port:
            optional["port"] = self.port

        if len(self._checks) > 0:
            optional["check"] = self._checks[0].check

        consul_api.agent.service.register(
            self.name, service_id=self.id, address=self.address, tags=self.tags, **optional
        )

    def add_check(self, check):
        self._checks.append(check)

    def checks(self):
        return self._checks

    def has_checks(self):
        return len(self._checks) > 0

    def __eq__(self, other):
        return (
            isinstance(other, self.__class__)
            and self.id == other.id
            and self.name == other.name
            and self.port == other.port
            and self.tags == other.tags
        )

    def __ne__(self, other):
        return not self.__eq__(other)

    def to_dict(self):
        data = {"id": self.id, "name": self.name}
        if self.port:
            data["port"] = self.port
        if self.tags and len(self.tags) > 0:
            data["tags"] = self.tags
        if len(self._checks) > 0:
            data["check"] = self._checks[0].to_dict()
        return data


class ConsulCheck:
    def __init__(
        self,
        check_id,
        name,
        node=None,
        host="localhost",
        script=None,
        interval=None,
        ttl=None,
        notes=None,
        tcp=None,
        http=None,
        timeout=None,
        service_id=None,
    ):
        self.check_id = self.name = name
        if check_id:
            self.check_id = check_id
        self.service_id = service_id
        self.notes = notes
        self.node = node
        self.host = host

        self.interval = self.validate_duration("interval", interval)
        self.ttl = self.validate_duration("ttl", ttl)
        self.script = script
        self.tcp = tcp
        self.http = http
        self.timeout = self.validate_duration("timeout", timeout)

        self.check = None

        if script:
            self.check = consul.Check.script(script, self.interval)

        if ttl:
            self.check = consul.Check.ttl(self.ttl)

        if http:
            self.check = consul.Check.http(http, self.interval, self.timeout)

        if tcp:
            regex = r"(?P<host>.*):(?P<port>(?:[0-9]+))$"
            match = re.match(regex, tcp)

            if not match:
                raise Exception("tcp check must be in host:port format")

            self.check = consul.Check.tcp(match.group("host").strip("[]"), int(match.group("port")), self.interval)

    def validate_duration(self, name, duration):
        if duration:
            duration_units = ["ns", "us", "ms", "s", "m", "h"]
            if not any(duration.endswith(suffix) for suffix in duration_units):
                duration = f"{duration}s"
        return duration

    def register(self, consul_api):
        consul_api.agent.check.register(
            self.name, check_id=self.check_id, service_id=self.service_id, notes=self.notes, check=self.check
        )

    def __eq__(self, other):
        return (
            isinstance(other, self.__class__)
            and self.check_id == other.check_id
            and self.service_id == other.service_id
            and self.name == other.name
            and self.script == other.script
            and self.interval == other.interval
        )

    def __ne__(self, other):
        return not self.__eq__(other)

    def to_dict(self):
        data = {}
        self._add(data, "id", attr="check_id")
        self._add(data, "name", attr="check_name")
        self._add(data, "script")
        self._add(data, "node")
        self._add(data, "notes")
        self._add(data, "host")
        self._add(data, "interval")
        self._add(data, "ttl")
        self._add(data, "tcp")
        self._add(data, "http")
        self._add(data, "timeout")
        self._add(data, "service_id")
        return data

    def _add(self, data, key, attr=None):
        try:
            if attr is None:
                attr = key
            data[key] = getattr(self, attr)
        except Exception:
            pass


def test_dependencies(module):
    if not python_consul_installed:
        module.fail_json(
            msg="py-consul required for this module. see https://github.com/criteo/py-consul?tab=readme-ov-file#installation"
        )


def main():
    module = AnsibleModule(
        argument_spec=dict(
            host=dict(default="localhost"),
            port=dict(default=8500, type="int"),
            scheme=dict(default="http"),
            validate_certs=dict(default=True, type="bool"),
            check_id=dict(),
            check_name=dict(),
            check_node=dict(),
            check_host=dict(),
            notes=dict(),
            script=dict(),
            service_id=dict(),
            service_name=dict(),
            service_address=dict(type="str"),
            service_port=dict(type="int"),
            state=dict(default="present", choices=["present", "absent"]),
            interval=dict(type="str"),
            ttl=dict(type="str"),
            tcp=dict(type="str"),
            http=dict(type="str"),
            timeout=dict(type="str"),
            tags=dict(type="list", elements="str"),
            token=dict(no_log=True),
        ),
        mutually_exclusive=[
            ("script", "ttl", "tcp", "http"),
        ],
        required_if=[
            ("state", "present", ["service_name"]),
            ("state", "absent", ["service_id", "service_name", "check_id", "check_name"], True),
        ],
        required_by={
            "script": "interval",
            "http": "interval",
            "tcp": "interval",
        },
        supports_check_mode=False,
    )
    p = module.params

    test_dependencies(module)
    if p["state"] == "absent" and any(p[x] for x in ["script", "ttl", "tcp", "http", "interval"]):
        module.fail_json(
            msg="The use of parameters 'script', 'ttl', 'tcp', 'http', 'interval' along with 'state=absent' is no longer allowed."
        )

    try:
        register_with_consul(module)
    except SystemExit:
        raise
    except ConnectionError as e:
        module.fail_json(msg=f"Could not connect to consul agent at {p['host']}:{p['port']}, error was {e}")
    except Exception as e:
        module.fail_json(msg=str(e))


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Aaron Smith <ajsmith10381@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: config_recorder
version_added: 1.0.0
short_description: Manage AWS Config Recorders
description:
  - Module manages AWS Config configuration recorder settings.
  - Prior to release 5.0.0 this module was called C(community.aws.aws_config_recorder).
    The usage did not change.
author:
  - "Aaron Smith (@slapula)"
options:
  name:
    description:
      - The name of the AWS Config resource.
    required: true
    type: str
  state:
    description:
      - Whether the Config rule should be present or absent.
    default: present
    choices: ['present', 'absent']
    type: str
  role_arn:
    description:
      - Amazon Resource Name (ARN) of the IAM role used to describe the AWS resources associated with the account.
      - Required when I(state=present).
    type: str
  recording_group:
    description:
      - Specifies the types of AWS resources for which AWS Config records configuration changes.
      - Required when I(state=present)
    suboptions:
      all_supported:
        description:
          - Specifies whether AWS Config records configuration changes for every supported type of regional resource.
          - If I(all_supported=true), when AWS Config adds support for a new type of regional resource, it starts
            recording resources of that type automatically.
          - If I(all_supported=true), you cannot enumerate a list of I(resource_types).
      include_global_types:
        description:
          - Specifies whether AWS Config includes all supported types of global resources (for example, IAM resources)
            with the resources that it records.
          - The configuration details for any global resource are the same in all regions. To prevent duplicate configuration items,
            you should consider customizing AWS Config in only one region to record global resources.
          - If you set I(include_global_types=true), you must also set I(all_supported=true).
          - If you set I(include_global_types=true), when AWS Config adds support for a new type of global resource, it starts recording
            resources of that type automatically.
      resource_types:
        description:
          - A list that specifies the types of AWS resources for which AWS Config records configuration changes (for example,
            C(AWS::EC2::Instance) or C(AWS::CloudTrail::Trail)).
          - Before you can set this option, you must set I(all_supported=false).
    type: dict
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: Create Configuration Recorder for AWS Config
  community.aws.config_recorder:
    name: test_configuration_recorder
    state: present
    role_arn: 'arn:aws:iam::123456789012:role/AwsConfigRecorder'
    recording_group:
      all_supported: true
      include_global_types: true
"""

RETURN = r"""#"""


try:
    import botocore
except ImportError:
    pass  # handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


def resource_exists(client, module, params):
    try:
        recorder = client.describe_configuration_recorders(ConfigurationRecorderNames=[params["name"]])
        return recorder["ConfigurationRecorders"][0]
    except is_boto3_error_code("NoSuchConfigurationRecorderException"):
        return
    except (
        botocore.exceptions.ClientError,
        botocore.exceptions.BotoCoreError,
    ) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e)


def create_resource(client, module, params, result):
    try:
        client.put_configuration_recorder(ConfigurationRecorder=params)
        result["changed"] = True
        result["recorder"] = camel_dict_to_snake_dict(resource_exists(client, module, params))
        return result
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Couldn't create AWS Config configuration recorder")


def update_resource(client, module, params, result):
    current_params = client.describe_configuration_recorders(ConfigurationRecorderNames=[params["name"]])

    if params != current_params["ConfigurationRecorders"][0]:
        try:
            client.put_configuration_recorder(ConfigurationRecorder=params)
            result["changed"] = True
            result["recorder"] = camel_dict_to_snake_dict(resource_exists(client, module, params))
            return result
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg="Couldn't update AWS Config configuration recorder")


def delete_resource(client, module, params, result):
    try:
        client.delete_configuration_recorder(ConfigurationRecorderName=params["name"])
        result["changed"] = True
        return result
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Couldn't delete AWS Config configuration recorder")


def main():
    module = AnsibleAWSModule(
        argument_spec={
            "name": dict(type="str", required=True),
            "state": dict(type="str", choices=["present", "absent"], default="present"),
            "role_arn": dict(type="str"),
            "recording_group": dict(type="dict"),
        },
        supports_check_mode=False,
        required_if=[
            ("state", "present", ["role_arn", "recording_group"]),
        ],
    )

    result = {"changed": False}

    name = module.params.get("name")
    state = module.params.get("state")

    params = {}
    if name:
        params["name"] = name
    if module.params.get("role_arn"):
        params["roleARN"] = module.params.get("role_arn")
    if module.params.get("recording_group"):
        params["recordingGroup"] = {}
        if module.params.get("recording_group").get("all_supported") is not None:
            params["recordingGroup"].update({"allSupported": module.params.get("recording_group").get("all_supported")})
        if module.params.get("recording_group").get("include_global_types") is not None:
            params["recordingGroup"].update(
                {"includeGlobalResourceTypes": module.params.get("recording_group").get("include_global_types")}
            )
        if module.params.get("recording_group").get("resource_types"):
            params["recordingGroup"].update(
                {"resourceTypes": module.params.get("recording_group").get("resource_types")}
            )
        else:
            params["recordingGroup"].update({"resourceTypes": []})

    client = module.client("config", retry_decorator=AWSRetry.jittered_backoff())

    resource_status = resource_exists(client, module, params)

    if state == "present":
        if not resource_status:
            create_resource(client, module, params, result)
        if resource_status:
            update_resource(client, module, params, result)

    if state == "absent":
        if resource_status:
            delete_resource(client, module, params, result)

    module.exit_json(changed=result["changed"])


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_link_aggregation_policy
short_description: Link Aggregation Policy configuration for Cisco Intersight
description:
  - Manages Link Aggregation Policy configuration on Cisco Intersight.
  - A policy to configure LACP (Link Aggregation Control Protocol) settings for link aggregation on Cisco Intersight managed fabric interconnects.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs/fabric/LinkAggregationPolicies/get/).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Profiles, Policies, and Pools that are created within a Custom Organization are applicable only to devices in the same Organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the Link Aggregation Policy.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the Link Aggregation Policy.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  suspend_individual:
    description:
      - Flag tells the switch whether to suspend the port if it didn't receive LACP PDU.
      - If enabled, the switch will suspend the port in the port channel if LACP PDUs are not received.
    type: bool
    default: false
  lacp_rate:
    description:
      - Flag used to indicate whether LACP PDUs are to be sent 'fast', i.e., every 1 second.
      - C(normal) - LACP PDUs are sent every 30 seconds.
      - C(fast) - LACP PDUs are sent every 1 second.
    type: str
    choices: [normal, fast]
    default: normal
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create a Link Aggregation Policy with default settings
  cisco.intersight.intersight_link_aggregation_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "LinkAgg-Policy-01"
    description: "Link aggregation policy with default LACP settings"
    tags:
      - Key: "Site"
        Value: "DataCenter-A"
    state: present

- name: Create a Link Aggregation Policy with fast LACP rate
  cisco.intersight.intersight_link_aggregation_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "LinkAgg-Fast-Policy"
    description: "Link aggregation policy with fast LACP rate"
    lacp_rate: fast
    suspend_individual: true
    state: present

- name: Create a Link Aggregation Policy with normal LACP rate and suspend individual disabled
  cisco.intersight.intersight_link_aggregation_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "LinkAgg-Normal-Policy"
    lacp_rate: normal
    suspend_individual: false
    state: present

- name: Update a Link Aggregation Policy
  cisco.intersight.intersight_link_aggregation_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "LinkAgg-Policy-01"
    description: "Updated link aggregation policy"
    lacp_rate: fast
    suspend_individual: true
    state: present

- name: Delete a Link Aggregation Policy
  cisco.intersight.intersight_link_aggregation_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "LinkAgg-Policy-01"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "LinkAgg-Policy-01",
        "ObjectType": "fabric.LinkAggregationPolicy",
        "SuspendIndividual": false,
        "LacpRate": "normal",
        "Tags": [
            {
                "Key": "Site",
                "Value": "DataCenter-A"
            }
        ]
    }
'''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        suspend_individual=dict(type='bool', default=False),
        lacp_rate=dict(type='str', choices=['normal', 'fast'], default='normal')
    )
    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True
    )

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    # Resource path used to configure policy
    resource_path = '/fabric/LinkAggregationPolicies'

    # Define API body used in compares or create
    intersight.api_body = {
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
        'Name': intersight.module.params['name'],
    }

    if intersight.module.params['state'] == 'present':
        intersight.set_tags_and_description()
        intersight.api_body['SuspendIndividual'] = intersight.module.params['suspend_individual']
        intersight.api_body['LacpRate'] = intersight.module.params['lacp_rate']

    # Configure the policy
    intersight.configure_policy_or_profile(resource_path=resource_path)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()
